"""The horizontal commands module.

These commands are used in the following models:
DPO7AX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - HORizontal:ACQDURATION?
    - HORizontal:DELay:MODe {ON|OFF|<NR1>}
    - HORizontal:DELay:MODe?
    - HORizontal:DELay:TIMe <NR3>
    - HORizontal:DELay:TIMe?
    - HORizontal:DIVisions?
    - HORizontal:MAIn:INTERPRatio?
    - HORizontal:MODe {AUTO|MANual}
    - HORizontal:MODe:MANual:CONFIGure {HORIZontalscale|RECORDLength}
    - HORizontal:MODe:MANual:CONFIGure?
    - HORizontal:MODe:RECOrdlength <NR1>
    - HORizontal:MODe:RECOrdlength?
    - HORizontal:MODe:SAMPLERate <NR1>
    - HORizontal:MODe:SAMPLERate?
    - HORizontal:MODe:SCAle <NR1>
    - HORizontal:MODe:SCAle?
    - HORizontal:MODe?
    - HORizontal:POSition <NR3>
    - HORizontal:POSition?
    - HORizontal:PREViewstate?
    - HORizontal:RECOrdlength <NR1>
    - HORizontal:RECOrdlength?
    - HORizontal:ROLL?
    - HORizontal:SAMPLERate <NR3>
    - HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide {ON|OFF|1|0}
    - HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide?
    - HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue {AUTOmatic|<NR3>}
    - HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue?
    - HORizontal:SAMPLERate?
    - HORizontal:SCAle <NR3>
    - HORizontal:SCAle?
    - HORizontal?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class HorizontalScale(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:SCAle`` command.

    Description:
        - This command sets or queries the horizontal scale.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:SCAle?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:SCAle?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:SCAle value`` command.

    SCPI Syntax:
        ```
        - HORizontal:SCAle <NR3>
        - HORizontal:SCAle?
        ```

    Info:
        - ``<NR3>`` is the horizontal scale in time per division.
    """


class HorizontalSamplerateAnalyzemodeMinimumValue(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue`` command.

    Description:
        - Sets or queries the minimum sample rate used by Analysis Automatic horizontal mode.

    Usage:
        - Using the ``.query()`` method will send the
          ``HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue?`` query.
        - Using the ``.verify(value)`` method will send the
          ``HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue?`` query and raise an AssertionError if
          the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue value`` command.

    SCPI Syntax:
        ```
        - HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue {AUTOmatic|<NR3>}
        - HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue?
        ```

    Info:
        - ``AUTOmatic`` allows the instrument to set the minimum value.
        - ``<NR3>`` is the minimum sample rate.
    """


class HorizontalSamplerateAnalyzemodeMinimumOverride(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide`` command.

    Description:
        - Sets or queries the flag which allows override of the horizontal analyze minimum sample
          rate.

    Usage:
        - Using the ``.query()`` method will send the
          ``HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide?`` query.
        - Using the ``.verify(value)`` method will send the
          ``HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide?`` query and raise an AssertionError
          if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide value`` command.

    SCPI Syntax:
        ```
        - HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide {ON|OFF|1|0}
        - HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide?
        ```

    Info:
        - ``0`` does not allow override of the horizontal analyze minimum sample rate.
        - ``1`` allows override of the horizontal analyze minimum sample rate.
        - ``OFF`` does not allow override of the horizontal analyze minimum sample rate.
        - ``ON`` allows override of the horizontal analyze minimum sample rate.
    """


class HorizontalSamplerateAnalyzemodeMinimum(SCPICmdRead):
    """The ``HORizontal:SAMPLERate:ANALYZemode:MINimum`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:SAMPLERate:ANALYZemode:MINimum?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``HORizontal:SAMPLERate:ANALYZemode:MINimum?`` query and raise an AssertionError if the
          returned value does not match ``value``.

    Properties:
        - ``.override``: The ``HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide`` command.
        - ``.value``: The ``HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._override = HorizontalSamplerateAnalyzemodeMinimumOverride(
            device, f"{self._cmd_syntax}:OVERRide"
        )
        self._value = HorizontalSamplerateAnalyzemodeMinimumValue(
            device, f"{self._cmd_syntax}:VALue"
        )

    @property
    def override(self) -> HorizontalSamplerateAnalyzemodeMinimumOverride:
        """Return the ``HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide`` command.

        Description:
            - Sets or queries the flag which allows override of the horizontal analyze minimum
              sample rate.

        Usage:
            - Using the ``.query()`` method will send the
              ``HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide?`` query.
            - Using the ``.verify(value)`` method will send the
              ``HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide value`` command.

        SCPI Syntax:
            ```
            - HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide {ON|OFF|1|0}
            - HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide?
            ```

        Info:
            - ``0`` does not allow override of the horizontal analyze minimum sample rate.
            - ``1`` allows override of the horizontal analyze minimum sample rate.
            - ``OFF`` does not allow override of the horizontal analyze minimum sample rate.
            - ``ON`` allows override of the horizontal analyze minimum sample rate.
        """
        return self._override

    @property
    def value(self) -> HorizontalSamplerateAnalyzemodeMinimumValue:
        """Return the ``HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue`` command.

        Description:
            - Sets or queries the minimum sample rate used by Analysis Automatic horizontal mode.

        Usage:
            - Using the ``.query()`` method will send the
              ``HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue?`` query.
            - Using the ``.verify(value)`` method will send the
              ``HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue?`` query and raise an AssertionError
              if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue value`` command.

        SCPI Syntax:
            ```
            - HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue {AUTOmatic|<NR3>}
            - HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue?
            ```

        Info:
            - ``AUTOmatic`` allows the instrument to set the minimum value.
            - ``<NR3>`` is the minimum sample rate.
        """
        return self._value


class HorizontalSamplerateAnalyzemode(SCPICmdRead):
    """The ``HORizontal:SAMPLERate:ANALYZemode`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:SAMPLERate:ANALYZemode?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:SAMPLERate:ANALYZemode?``
          query and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.minimum``: The ``HORizontal:SAMPLERate:ANALYZemode:MINimum`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._minimum = HorizontalSamplerateAnalyzemodeMinimum(
            device, f"{self._cmd_syntax}:MINimum"
        )

    @property
    def minimum(self) -> HorizontalSamplerateAnalyzemodeMinimum:
        """Return the ``HORizontal:SAMPLERate:ANALYZemode:MINimum`` command tree.

        Usage:
            - Using the ``.query()`` method will send the
              ``HORizontal:SAMPLERate:ANALYZemode:MINimum?`` query.
            - Using the ``.verify(value)`` method will send the
              ``HORizontal:SAMPLERate:ANALYZemode:MINimum?`` query and raise an AssertionError if
              the returned value does not match ``value``.

        Sub-properties:
            - ``.override``: The ``HORizontal:SAMPLERate:ANALYZemode:MINimum:OVERRide`` command.
            - ``.value``: The ``HORizontal:SAMPLERate:ANALYZemode:MINimum:VALue`` command.
        """
        return self._minimum


class HorizontalSamplerate(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:SAMPLERate`` command.

    Description:
        - This command sets or queries the horizontal sample rate.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:SAMPLERate?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:SAMPLERate?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:SAMPLERate value`` command.

    SCPI Syntax:
        ```
        - HORizontal:SAMPLERate <NR3>
        - HORizontal:SAMPLERate?
        ```

    Info:
        - ``<NR3>`` is the horizontal sample rate in samples per second.

    Properties:
        - ``.analyzemode``: The ``HORizontal:SAMPLERate:ANALYZemode`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._analyzemode = HorizontalSamplerateAnalyzemode(
            device, f"{self._cmd_syntax}:ANALYZemode"
        )

    @property
    def analyzemode(self) -> HorizontalSamplerateAnalyzemode:
        """Return the ``HORizontal:SAMPLERate:ANALYZemode`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:SAMPLERate:ANALYZemode?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``HORizontal:SAMPLERate:ANALYZemode?`` query and raise an AssertionError if the
              returned value does not match ``value``.

        Sub-properties:
            - ``.minimum``: The ``HORizontal:SAMPLERate:ANALYZemode:MINimum`` command tree.
        """
        return self._analyzemode


class HorizontalRoll(SCPICmdRead):
    """The ``HORizontal:ROLL`` command.

    Description:
        - Queries the horizontal roll mode status.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:ROLL?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:ROLL?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - HORizontal:ROLL?
        ```
    """


class HorizontalRecordlength(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:RECOrdlength`` command.

    Description:
        - This command sets or queries the horizontal record length. To change the record length the
          Horizontal Mode must be set to Manual.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:RECOrdlength?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:RECOrdlength?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:RECOrdlength value``
          command.

    SCPI Syntax:
        ```
        - HORizontal:RECOrdlength <NR1>
        - HORizontal:RECOrdlength?
        ```

    Info:
        - ``<NR1>`` is the horizontal record length.
    """


class HorizontalPreviewstate(SCPICmdRead):
    """The ``HORizontal:PREViewstate`` command.

    Description:
        - This query returns the display system preview state.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:PREViewstate?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:PREViewstate?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - HORizontal:PREViewstate?
        ```
    """


class HorizontalPosition(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:POSition`` command.

    Description:
        - This command sets or queries the horizontal position as a percent of screen width. When
          Horizontal Delay Mode is turned off, this command is equivalent to adjusting the
          HORIZONTAL POSITION knob on the front panel. When Horizontal Delay Mode is turned on, the
          horizontal position is forced to 50%.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:POSition?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:POSition?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:POSition value`` command.

    SCPI Syntax:
        ```
        - HORizontal:POSition <NR3>
        - HORizontal:POSition?
        ```

    Info:
        - ``<NR3>`` is from 0 to ≈100 and is the position of the trigger point on the screen (0 =
          left edge, 100 = right edge).
    """


class HorizontalModeScale(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:MODe:SCAle`` command.

    Description:
        - This command sets or queries the horizontal scale.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:MODe:SCAle?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:MODe:SCAle?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:MODe:SCAle value`` command.

    SCPI Syntax:
        ```
        - HORizontal:MODe:SCAle <NR1>
        - HORizontal:MODe:SCAle?
        ```

    Info:
        - ``<NR1>`` is the horizontal scale in seconds per division.
    """


class HorizontalModeSamplerate(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:MODe:SAMPLERate`` command.

    Description:
        - This command sets or queries the sample rate.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:MODe:SAMPLERate?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:MODe:SAMPLERate?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:MODe:SAMPLERate value``
          command.

    SCPI Syntax:
        ```
        - HORizontal:MODe:SAMPLERate <NR1>
        - HORizontal:MODe:SAMPLERate?
        ```

    Info:
        - ``<NR1>`` is the sample rate in samples per second.
    """


class HorizontalModeRecordlength(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:MODe:RECOrdlength`` command.

    Description:
        - This command sets or queries the record length.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:MODe:RECOrdlength?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:MODe:RECOrdlength?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:MODe:RECOrdlength value``
          command.

    SCPI Syntax:
        ```
        - HORizontal:MODe:RECOrdlength <NR1>
        - HORizontal:MODe:RECOrdlength?
        ```

    Info:
        - ``<NR1>`` is the record length in samples. Manual mode lets you change the record length,
          while the record length is read only for Automatic mode.
    """


class HorizontalModeManualConfigure(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:MODe:MANual:CONFIGure`` command.

    Description:
        - Sets or queries which horizontal control (scale or record length) will primarily change
          when the sample rate is changed in Manual mode. If the selected control (scale or record
          length) reaches a limit then the unselected control (record length or scale) may also
          change.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:MODe:MANual:CONFIGure?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:MODe:MANual:CONFIGure?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``HORizontal:MODe:MANual:CONFIGure value`` command.

    SCPI Syntax:
        ```
        - HORizontal:MODe:MANual:CONFIGure {HORIZontalscale|RECORDLength}
        - HORizontal:MODe:MANual:CONFIGure?
        ```

    Info:
        - ``HORIZontalscale`` will change when sample rate is adjusted.
        - ``RECORDLength`` will change when sample rate is adjusted.
    """


class HorizontalModeManual(SCPICmdRead):
    """The ``HORizontal:MODe:MANual`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:MODe:MANual?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:MODe:MANual?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.configure``: The ``HORizontal:MODe:MANual:CONFIGure`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._configure = HorizontalModeManualConfigure(device, f"{self._cmd_syntax}:CONFIGure")

    @property
    def configure(self) -> HorizontalModeManualConfigure:
        """Return the ``HORizontal:MODe:MANual:CONFIGure`` command.

        Description:
            - Sets or queries which horizontal control (scale or record length) will primarily
              change when the sample rate is changed in Manual mode. If the selected control (scale
              or record length) reaches a limit then the unselected control (record length or scale)
              may also change.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:MODe:MANual:CONFIGure?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``HORizontal:MODe:MANual:CONFIGure?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``HORizontal:MODe:MANual:CONFIGure value`` command.

        SCPI Syntax:
            ```
            - HORizontal:MODe:MANual:CONFIGure {HORIZontalscale|RECORDLength}
            - HORizontal:MODe:MANual:CONFIGure?
            ```

        Info:
            - ``HORIZontalscale`` will change when sample rate is adjusted.
            - ``RECORDLength`` will change when sample rate is adjusted.
        """
        return self._configure


class HorizontalMode(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:MODe`` command.

    Description:
        - This command set or queries the horizontal operating mode.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:MODe?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:MODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:MODe value`` command.

    SCPI Syntax:
        ```
        - HORizontal:MODe {AUTO|MANual}
        - HORizontal:MODe?
        ```

    Info:
        - ``AUTO`` selects the automatic horizontal model. Auto mode automatically adjusts the
          sample rate and record length to provide a high acquisition rate in Fast Acq or signal
          fidelity in analysis. Record length is read only.
        - ``MANual`` selects the manual horizontal model. Manual mode lets you change the sample
          rate, horizontal scale, and record length. These values interact. For example, when you
          change record length then the horizontal scale also changes.

    Properties:
        - ``.manual``: The ``HORizontal:MODe:MANual`` command tree.
        - ``.recordlength``: The ``HORizontal:MODe:RECOrdlength`` command.
        - ``.samplerate``: The ``HORizontal:MODe:SAMPLERate`` command.
        - ``.scale``: The ``HORizontal:MODe:SCAle`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._manual = HorizontalModeManual(device, f"{self._cmd_syntax}:MANual")
        self._recordlength = HorizontalModeRecordlength(device, f"{self._cmd_syntax}:RECOrdlength")
        self._samplerate = HorizontalModeSamplerate(device, f"{self._cmd_syntax}:SAMPLERate")
        self._scale = HorizontalModeScale(device, f"{self._cmd_syntax}:SCAle")

    @property
    def manual(self) -> HorizontalModeManual:
        """Return the ``HORizontal:MODe:MANual`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:MODe:MANual?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:MODe:MANual?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.configure``: The ``HORizontal:MODe:MANual:CONFIGure`` command.
        """
        return self._manual

    @property
    def recordlength(self) -> HorizontalModeRecordlength:
        """Return the ``HORizontal:MODe:RECOrdlength`` command.

        Description:
            - This command sets or queries the record length.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:MODe:RECOrdlength?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:MODe:RECOrdlength?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``HORizontal:MODe:RECOrdlength value`` command.

        SCPI Syntax:
            ```
            - HORizontal:MODe:RECOrdlength <NR1>
            - HORizontal:MODe:RECOrdlength?
            ```

        Info:
            - ``<NR1>`` is the record length in samples. Manual mode lets you change the record
              length, while the record length is read only for Automatic mode.
        """
        return self._recordlength

    @property
    def samplerate(self) -> HorizontalModeSamplerate:
        """Return the ``HORizontal:MODe:SAMPLERate`` command.

        Description:
            - This command sets or queries the sample rate.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:MODe:SAMPLERate?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:MODe:SAMPLERate?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:MODe:SAMPLERate value``
              command.

        SCPI Syntax:
            ```
            - HORizontal:MODe:SAMPLERate <NR1>
            - HORizontal:MODe:SAMPLERate?
            ```

        Info:
            - ``<NR1>`` is the sample rate in samples per second.
        """
        return self._samplerate

    @property
    def scale(self) -> HorizontalModeScale:
        """Return the ``HORizontal:MODe:SCAle`` command.

        Description:
            - This command sets or queries the horizontal scale.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:MODe:SCAle?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:MODe:SCAle?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:MODe:SCAle value``
              command.

        SCPI Syntax:
            ```
            - HORizontal:MODe:SCAle <NR1>
            - HORizontal:MODe:SCAle?
            ```

        Info:
            - ``<NR1>`` is the horizontal scale in seconds per division.
        """
        return self._scale


class HorizontalMainInterpratio(SCPICmdRead):
    """The ``HORizontal:MAIn:INTERPRatio`` command.

    Description:
        - This query-only command returns the Horizontal interpolation ratio.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:MAIn:INTERPRatio?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:MAIn:INTERPRatio?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - HORizontal:MAIn:INTERPRatio?
        ```
    """


class HorizontalMain(SCPICmdRead):
    """The ``HORizontal:MAIn`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:MAIn?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:MAIn?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.interpratio``: The ``HORizontal:MAIn:INTERPRatio`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._interpratio = HorizontalMainInterpratio(device, f"{self._cmd_syntax}:INTERPRatio")

    @property
    def interpratio(self) -> HorizontalMainInterpratio:
        """Return the ``HORizontal:MAIn:INTERPRatio`` command.

        Description:
            - This query-only command returns the Horizontal interpolation ratio.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:MAIn:INTERPRatio?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:MAIn:INTERPRatio?``
              query and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - HORizontal:MAIn:INTERPRatio?
            ```
        """
        return self._interpratio


class HorizontalDivisions(SCPICmdRead):
    """The ``HORizontal:DIVisions`` command.

    Description:
        - This query-only command returns the number of graticule divisions.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:DIVisions?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:DIVisions?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - HORizontal:DIVisions?
        ```
    """


class HorizontalDelayTime(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:DELay:TIMe`` command.

    Description:
        - This command sets or queries the horizontal delay time that is used when delay mode is on.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:DELay:TIMe?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:DELay:TIMe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:DELay:TIMe value`` command.

    SCPI Syntax:
        ```
        - HORizontal:DELay:TIMe <NR3>
        - HORizontal:DELay:TIMe?
        ```

    Info:
        - ``<NR3>`` is the delay in seconds.
    """


class HorizontalDelayMode(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:DELay:MODe`` command.

    Description:
        - This command sets or queries the horizontal delay mode.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:DELay:MODe?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:DELay:MODe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:DELay:MODe value`` command.

    SCPI Syntax:
        ```
        - HORizontal:DELay:MODe {ON|OFF|<NR1>}
        - HORizontal:DELay:MODe?
        ```

    Info:
        - ``OFF`` sets the Horizontal Delay Mode to off. This causes the ``HORizontal:POSition``
          command to operate like the HORIZONTAL POSITION knob on the front panel.
        - ``ON`` sets the Horizontal Delay Mode to on. This causes the ``HORizontal:DELay:TIMe``
          command to operate like the HORIZONTAL POSITION knob on the front panel.
        - ``<NR1> = 0`` sets the Horizontal Delay Mode to off; any other value sets this mode to on.
    """


class HorizontalDelay(SCPICmdRead):
    """The ``HORizontal:DELay`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:DELay?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:DELay?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.mode``: The ``HORizontal:DELay:MODe`` command.
        - ``.time``: The ``HORizontal:DELay:TIMe`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._mode = HorizontalDelayMode(device, f"{self._cmd_syntax}:MODe")
        self._time = HorizontalDelayTime(device, f"{self._cmd_syntax}:TIMe")

    @property
    def mode(self) -> HorizontalDelayMode:
        """Return the ``HORizontal:DELay:MODe`` command.

        Description:
            - This command sets or queries the horizontal delay mode.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:DELay:MODe?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:DELay:MODe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:DELay:MODe value``
              command.

        SCPI Syntax:
            ```
            - HORizontal:DELay:MODe {ON|OFF|<NR1>}
            - HORizontal:DELay:MODe?
            ```

        Info:
            - ``OFF`` sets the Horizontal Delay Mode to off. This causes the ``HORizontal:POSition``
              command to operate like the HORIZONTAL POSITION knob on the front panel.
            - ``ON`` sets the Horizontal Delay Mode to on. This causes the ``HORizontal:DELay:TIMe``
              command to operate like the HORIZONTAL POSITION knob on the front panel.
            - ``<NR1> = 0`` sets the Horizontal Delay Mode to off; any other value sets this mode to
              on.
        """
        return self._mode

    @property
    def time(self) -> HorizontalDelayTime:
        """Return the ``HORizontal:DELay:TIMe`` command.

        Description:
            - This command sets or queries the horizontal delay time that is used when delay mode is
              on.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:DELay:TIMe?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:DELay:TIMe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:DELay:TIMe value``
              command.

        SCPI Syntax:
            ```
            - HORizontal:DELay:TIMe <NR3>
            - HORizontal:DELay:TIMe?
            ```

        Info:
            - ``<NR3>`` is the delay in seconds.
        """
        return self._time


class HorizontalAcqduration(SCPICmdRead):
    """The ``HORizontal:ACQDURATION`` command.

    Description:
        - This query returns the timebase duration.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:ACQDURATION?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:ACQDURATION?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - HORizontal:ACQDURATION?
        ```
    """


#  pylint: disable=too-many-instance-attributes
class Horizontal(SCPICmdRead):
    """The ``HORizontal`` command.

    Description:
        - Queries the current horizontal settings.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - HORizontal?
        ```

    Properties:
        - ``.acqduration``: The ``HORizontal:ACQDURATION`` command.
        - ``.delay``: The ``HORizontal:DELay`` command tree.
        - ``.divisions``: The ``HORizontal:DIVisions`` command.
        - ``.main``: The ``HORizontal:MAIn`` command tree.
        - ``.mode``: The ``HORizontal:MODe`` command.
        - ``.position``: The ``HORizontal:POSition`` command.
        - ``.previewstate``: The ``HORizontal:PREViewstate`` command.
        - ``.recordlength``: The ``HORizontal:RECOrdlength`` command.
        - ``.roll``: The ``HORizontal:ROLL`` command.
        - ``.samplerate``: The ``HORizontal:SAMPLERate`` command.
        - ``.scale``: The ``HORizontal:SCAle`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "HORizontal"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._acqduration = HorizontalAcqduration(device, f"{self._cmd_syntax}:ACQDURATION")
        self._delay = HorizontalDelay(device, f"{self._cmd_syntax}:DELay")
        self._divisions = HorizontalDivisions(device, f"{self._cmd_syntax}:DIVisions")
        self._main = HorizontalMain(device, f"{self._cmd_syntax}:MAIn")
        self._mode = HorizontalMode(device, f"{self._cmd_syntax}:MODe")
        self._position = HorizontalPosition(device, f"{self._cmd_syntax}:POSition")
        self._previewstate = HorizontalPreviewstate(device, f"{self._cmd_syntax}:PREViewstate")
        self._recordlength = HorizontalRecordlength(device, f"{self._cmd_syntax}:RECOrdlength")
        self._roll = HorizontalRoll(device, f"{self._cmd_syntax}:ROLL")
        self._samplerate = HorizontalSamplerate(device, f"{self._cmd_syntax}:SAMPLERate")
        self._scale = HorizontalScale(device, f"{self._cmd_syntax}:SCAle")

    @property
    def acqduration(self) -> HorizontalAcqduration:
        """Return the ``HORizontal:ACQDURATION`` command.

        Description:
            - This query returns the timebase duration.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:ACQDURATION?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:ACQDURATION?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - HORizontal:ACQDURATION?
            ```
        """
        return self._acqduration

    @property
    def delay(self) -> HorizontalDelay:
        """Return the ``HORizontal:DELay`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:DELay?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:DELay?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.mode``: The ``HORizontal:DELay:MODe`` command.
            - ``.time``: The ``HORizontal:DELay:TIMe`` command.
        """
        return self._delay

    @property
    def divisions(self) -> HorizontalDivisions:
        """Return the ``HORizontal:DIVisions`` command.

        Description:
            - This query-only command returns the number of graticule divisions.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:DIVisions?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:DIVisions?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - HORizontal:DIVisions?
            ```
        """
        return self._divisions

    @property
    def main(self) -> HorizontalMain:
        """Return the ``HORizontal:MAIn`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:MAIn?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:MAIn?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.interpratio``: The ``HORizontal:MAIn:INTERPRatio`` command.
        """
        return self._main

    @property
    def mode(self) -> HorizontalMode:
        """Return the ``HORizontal:MODe`` command.

        Description:
            - This command set or queries the horizontal operating mode.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:MODe?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:MODe?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:MODe value`` command.

        SCPI Syntax:
            ```
            - HORizontal:MODe {AUTO|MANual}
            - HORizontal:MODe?
            ```

        Info:
            - ``AUTO`` selects the automatic horizontal model. Auto mode automatically adjusts the
              sample rate and record length to provide a high acquisition rate in Fast Acq or signal
              fidelity in analysis. Record length is read only.
            - ``MANual`` selects the manual horizontal model. Manual mode lets you change the sample
              rate, horizontal scale, and record length. These values interact. For example, when
              you change record length then the horizontal scale also changes.

        Sub-properties:
            - ``.manual``: The ``HORizontal:MODe:MANual`` command tree.
            - ``.recordlength``: The ``HORizontal:MODe:RECOrdlength`` command.
            - ``.samplerate``: The ``HORizontal:MODe:SAMPLERate`` command.
            - ``.scale``: The ``HORizontal:MODe:SCAle`` command.
        """
        return self._mode

    @property
    def position(self) -> HorizontalPosition:
        """Return the ``HORizontal:POSition`` command.

        Description:
            - This command sets or queries the horizontal position as a percent of screen width.
              When Horizontal Delay Mode is turned off, this command is equivalent to adjusting the
              HORIZONTAL POSITION knob on the front panel. When Horizontal Delay Mode is turned on,
              the horizontal position is forced to 50%.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:POSition?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:POSition?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:POSition value``
              command.

        SCPI Syntax:
            ```
            - HORizontal:POSition <NR3>
            - HORizontal:POSition?
            ```

        Info:
            - ``<NR3>`` is from 0 to ≈100 and is the position of the trigger point on the screen (0
              = left edge, 100 = right edge).
        """
        return self._position

    @property
    def previewstate(self) -> HorizontalPreviewstate:
        """Return the ``HORizontal:PREViewstate`` command.

        Description:
            - This query returns the display system preview state.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:PREViewstate?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:PREViewstate?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - HORizontal:PREViewstate?
            ```
        """
        return self._previewstate

    @property
    def recordlength(self) -> HorizontalRecordlength:
        """Return the ``HORizontal:RECOrdlength`` command.

        Description:
            - This command sets or queries the horizontal record length. To change the record length
              the Horizontal Mode must be set to Manual.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:RECOrdlength?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:RECOrdlength?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:RECOrdlength value``
              command.

        SCPI Syntax:
            ```
            - HORizontal:RECOrdlength <NR1>
            - HORizontal:RECOrdlength?
            ```

        Info:
            - ``<NR1>`` is the horizontal record length.
        """
        return self._recordlength

    @property
    def roll(self) -> HorizontalRoll:
        """Return the ``HORizontal:ROLL`` command.

        Description:
            - Queries the horizontal roll mode status.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:ROLL?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:ROLL?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - HORizontal:ROLL?
            ```
        """
        return self._roll

    @property
    def samplerate(self) -> HorizontalSamplerate:
        """Return the ``HORizontal:SAMPLERate`` command.

        Description:
            - This command sets or queries the horizontal sample rate.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:SAMPLERate?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:SAMPLERate?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:SAMPLERate value``
              command.

        SCPI Syntax:
            ```
            - HORizontal:SAMPLERate <NR3>
            - HORizontal:SAMPLERate?
            ```

        Info:
            - ``<NR3>`` is the horizontal sample rate in samples per second.

        Sub-properties:
            - ``.analyzemode``: The ``HORizontal:SAMPLERate:ANALYZemode`` command tree.
        """
        return self._samplerate

    @property
    def scale(self) -> HorizontalScale:
        """Return the ``HORizontal:SCAle`` command.

        Description:
            - This command sets or queries the horizontal scale.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:SCAle?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:SCAle?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:SCAle value`` command.

        SCPI Syntax:
            ```
            - HORizontal:SCAle <NR3>
            - HORizontal:SCAle?
            ```

        Info:
            - ``<NR3>`` is the horizontal scale in time per division.
        """
        return self._scale



"""The mark commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - MARK {NEXT|PREVious}
    - MARK:CREATE {CH<x>|MATH<x>|REF<x>|COLUMN}
    - MARK:DELEte {CH<x>|MATH<x>|SELECTED|REF<x>|ALL|COLUMN}
    - MARK:FREE?
    - MARK:SELECTED:END?
    - MARK:SELECTED:FOCUS?
    - MARK:SELECTED:LABel <QString>
    - MARK:SELECTED:LABel?
    - MARK:SELECTED:MARKSINCOLumn?
    - MARK:SELECTED:OWNer?
    - MARK:SELECTED:SOUrce?
    - MARK:SELECTED:STARt?
    - MARK:SELECTED:STATE {ON|OFF|<NR1>}
    - MARK:SELECTED:STATE?
    - MARK:SELECTED:ZOOm:POSition?
    - MARK:SELECTED:ZOOm:SCAle?
    - MARK:TOTal?
    - MARK?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class MarkTotal(SCPICmdRead):
    """The ``MARK:TOTal`` command.

    Description:
        - Returns how many marks are currently in use. There can be a total of 1,024 marks returned.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:TOTal?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:TOTal?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:TOTal?
        ```
    """


class MarkSelectedZoomScale(SCPICmdRead):
    """The ``MARK:SELECTED:ZOOm:SCAle`` command.

    Description:
        - This query-only command returns the scale of the selected mark of the zoom overview
          window. The returned value might be < 0 (for example from Search), which means that the
          zoom scale will not be changed when Next or Prev makes this the selected mark.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELECTED:ZOOm:SCAle?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:ZOOm:SCAle?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELECTED:ZOOm:SCAle?
        ```
    """


class MarkSelectedZoomPosition(SCPICmdRead):
    """The ``MARK:SELECTED:ZOOm:POSition`` command.

    Description:
        - This query-only command returns the position of the selected mark, of the zoom overview
          window. If the return value is less than zero then the zoom scale will not be changed when
          Next or Prev makes this the selected mark.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELECTED:ZOOm:POSition?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:ZOOm:POSition?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELECTED:ZOOm:POSition?
        ```
    """


class MarkSelectedZoom(SCPICmdRead):
    """The ``MARK:SELECTED:ZOOm`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELECTED:ZOOm?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:ZOOm?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.position``: The ``MARK:SELECTED:ZOOm:POSition`` command.
        - ``.scale``: The ``MARK:SELECTED:ZOOm:SCAle`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._position = MarkSelectedZoomPosition(device, f"{self._cmd_syntax}:POSition")
        self._scale = MarkSelectedZoomScale(device, f"{self._cmd_syntax}:SCAle")

    @property
    def position(self) -> MarkSelectedZoomPosition:
        """Return the ``MARK:SELECTED:ZOOm:POSition`` command.

        Description:
            - This query-only command returns the position of the selected mark, of the zoom
              overview window. If the return value is less than zero then the zoom scale will not be
              changed when Next or Prev makes this the selected mark.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELECTED:ZOOm:POSition?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:ZOOm:POSition?``
              query and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELECTED:ZOOm:POSition?
            ```
        """
        return self._position

    @property
    def scale(self) -> MarkSelectedZoomScale:
        """Return the ``MARK:SELECTED:ZOOm:SCAle`` command.

        Description:
            - This query-only command returns the scale of the selected mark of the zoom overview
              window. The returned value might be < 0 (for example from Search), which means that
              the zoom scale will not be changed when Next or Prev makes this the selected mark.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELECTED:ZOOm:SCAle?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:ZOOm:SCAle?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELECTED:ZOOm:SCAle?
            ```
        """
        return self._scale


class MarkSelectedState(SCPICmdWrite, SCPICmdRead):
    """The ``MARK:SELECTED:STATE`` command.

    Description:
        - This command sets or queries the on or off state of the selected mark. The selected mark
          is at or near the center of the display. If you push the Set/Clear button, this mark will
          disappear.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELECTED:STATE?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:STATE?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MARK:SELECTED:STATE value`` command.

    SCPI Syntax:
        ```
        - MARK:SELECTED:STATE {ON|OFF|<NR1>}
        - MARK:SELECTED:STATE?
        ```

    Info:
        - ``<NR1>`` = 0 disables the selected mark; any other value enables the selected mark. For
          queries, a 0 is returned if the selected mark state is off; a 1 is returned if the
          selected mark state is on.
        - ``OFF`` argument disables selected mark.
        - ``ON`` argument enables the selected mark.
    """


class MarkSelectedStart(SCPICmdRead):
    """The ``MARK:SELECTED:STARt`` command.

    Description:
        - This query-only command returns the starting point of the selected mark, 0 to 100% of the
          waveform.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELECTED:STARt?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:STARt?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELECTED:STARt?
        ```
    """


class MarkSelectedSource(SCPICmdRead):
    """The ``MARK:SELECTED:SOUrce`` command.

    Description:
        - This query-only command returns the source waveform for the selected mark.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELECTED:SOUrce?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:SOUrce?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELECTED:SOUrce?
        ```
    """


class MarkSelectedOwner(SCPICmdRead):
    """The ``MARK:SELECTED:OWNer`` command.

    Description:
        - This query-only command returns the owner of the selected mark.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELECTED:OWNer?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:OWNer?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELECTED:OWNer?
        ```
    """


class MarkSelectedMarksincolumn(SCPICmdRead):
    """The ``MARK:SELECTED:MARKSINCOLumn`` command.

    Description:
        - This query-only command returns the number of marks in the current zoom pixel column.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELECTED:MARKSINCOLumn?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:MARKSINCOLumn?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELECTED:MARKSINCOLumn?
        ```

    Info:
        - ``MARK:SELECTED:MARKSINCOLUMN?`` might return ``:MARK:SELECTED:MARKSINCOLUMN 1``,
          indicating there is 1 mark in the pixel column.
    """


class MarkSelectedLabel(SCPICmdWrite, SCPICmdRead):
    """The ``MARK:SELECTED:LABel`` command.

    Description:
        - This command sets or queries the label for the selected mark.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELECTED:LABel?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:LABel?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MARK:SELECTED:LABel value`` command.

    SCPI Syntax:
        ```
        - MARK:SELECTED:LABel <QString>
        - MARK:SELECTED:LABel?
        ```

    Info:
        - ``<QString>`` is the quoted string label for the mark.
    """

    _WRAP_ARG_WITH_QUOTES = True


class MarkSelectedFocus(SCPICmdRead):
    """The ``MARK:SELECTED:FOCUS`` command.

    Description:
        - This query-only command returns the focus of the selected mark, 0 to 100% of the waveform.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELECTED:FOCUS?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:FOCUS?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELECTED:FOCUS?
        ```
    """


class MarkSelectedEnd(SCPICmdRead):
    """The ``MARK:SELECTED:END`` command.

    Description:
        - This query-only command returns the end of the selected mark, 0 to 100% of the waveform.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELECTED:END?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:END?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:SELECTED:END?
        ```
    """


#  pylint: disable=too-many-instance-attributes
class MarkSelected(SCPICmdRead):
    """The ``MARK:SELECTED`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:SELECTED?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:SELECTED?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.end``: The ``MARK:SELECTED:END`` command.
        - ``.focus``: The ``MARK:SELECTED:FOCUS`` command.
        - ``.label``: The ``MARK:SELECTED:LABel`` command.
        - ``.marksincolumn``: The ``MARK:SELECTED:MARKSINCOLumn`` command.
        - ``.owner``: The ``MARK:SELECTED:OWNer`` command.
        - ``.source``: The ``MARK:SELECTED:SOUrce`` command.
        - ``.start``: The ``MARK:SELECTED:STARt`` command.
        - ``.state``: The ``MARK:SELECTED:STATE`` command.
        - ``.zoom``: The ``MARK:SELECTED:ZOOm`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._end = MarkSelectedEnd(device, f"{self._cmd_syntax}:END")
        self._focus = MarkSelectedFocus(device, f"{self._cmd_syntax}:FOCUS")
        self._label = MarkSelectedLabel(device, f"{self._cmd_syntax}:LABel")
        self._marksincolumn = MarkSelectedMarksincolumn(device, f"{self._cmd_syntax}:MARKSINCOLumn")
        self._owner = MarkSelectedOwner(device, f"{self._cmd_syntax}:OWNer")
        self._source = MarkSelectedSource(device, f"{self._cmd_syntax}:SOUrce")
        self._start = MarkSelectedStart(device, f"{self._cmd_syntax}:STARt")
        self._state = MarkSelectedState(device, f"{self._cmd_syntax}:STATE")
        self._zoom = MarkSelectedZoom(device, f"{self._cmd_syntax}:ZOOm")

    @property
    def end(self) -> MarkSelectedEnd:
        """Return the ``MARK:SELECTED:END`` command.

        Description:
            - This query-only command returns the end of the selected mark, 0 to 100% of the
              waveform.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELECTED:END?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:END?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELECTED:END?
            ```
        """
        return self._end

    @property
    def focus(self) -> MarkSelectedFocus:
        """Return the ``MARK:SELECTED:FOCUS`` command.

        Description:
            - This query-only command returns the focus of the selected mark, 0 to 100% of the
              waveform.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELECTED:FOCUS?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:FOCUS?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELECTED:FOCUS?
            ```
        """
        return self._focus

    @property
    def label(self) -> MarkSelectedLabel:
        """Return the ``MARK:SELECTED:LABel`` command.

        Description:
            - This command sets or queries the label for the selected mark.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELECTED:LABel?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:LABel?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MARK:SELECTED:LABel value``
              command.

        SCPI Syntax:
            ```
            - MARK:SELECTED:LABel <QString>
            - MARK:SELECTED:LABel?
            ```

        Info:
            - ``<QString>`` is the quoted string label for the mark.
        """
        return self._label

    @property
    def marksincolumn(self) -> MarkSelectedMarksincolumn:
        """Return the ``MARK:SELECTED:MARKSINCOLumn`` command.

        Description:
            - This query-only command returns the number of marks in the current zoom pixel column.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELECTED:MARKSINCOLumn?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:MARKSINCOLumn?``
              query and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELECTED:MARKSINCOLumn?
            ```

        Info:
            - ``MARK:SELECTED:MARKSINCOLUMN?`` might return ``:MARK:SELECTED:MARKSINCOLUMN 1``,
              indicating there is 1 mark in the pixel column.
        """
        return self._marksincolumn

    @property
    def owner(self) -> MarkSelectedOwner:
        """Return the ``MARK:SELECTED:OWNer`` command.

        Description:
            - This query-only command returns the owner of the selected mark.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELECTED:OWNer?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:OWNer?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELECTED:OWNer?
            ```
        """
        return self._owner

    @property
    def source(self) -> MarkSelectedSource:
        """Return the ``MARK:SELECTED:SOUrce`` command.

        Description:
            - This query-only command returns the source waveform for the selected mark.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELECTED:SOUrce?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:SOUrce?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELECTED:SOUrce?
            ```
        """
        return self._source

    @property
    def start(self) -> MarkSelectedStart:
        """Return the ``MARK:SELECTED:STARt`` command.

        Description:
            - This query-only command returns the starting point of the selected mark, 0 to 100% of
              the waveform.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELECTED:STARt?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:STARt?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:SELECTED:STARt?
            ```
        """
        return self._start

    @property
    def state(self) -> MarkSelectedState:
        """Return the ``MARK:SELECTED:STATE`` command.

        Description:
            - This command sets or queries the on or off state of the selected mark. The selected
              mark is at or near the center of the display. If you push the Set/Clear button, this
              mark will disappear.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELECTED:STATE?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:STATE?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``MARK:SELECTED:STATE value``
              command.

        SCPI Syntax:
            ```
            - MARK:SELECTED:STATE {ON|OFF|<NR1>}
            - MARK:SELECTED:STATE?
            ```

        Info:
            - ``<NR1>`` = 0 disables the selected mark; any other value enables the selected mark.
              For queries, a 0 is returned if the selected mark state is off; a 1 is returned if the
              selected mark state is on.
            - ``OFF`` argument disables selected mark.
            - ``ON`` argument enables the selected mark.
        """
        return self._state

    @property
    def zoom(self) -> MarkSelectedZoom:
        """Return the ``MARK:SELECTED:ZOOm`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELECTED:ZOOm?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELECTED:ZOOm?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.position``: The ``MARK:SELECTED:ZOOm:POSition`` command.
            - ``.scale``: The ``MARK:SELECTED:ZOOm:SCAle`` command.
        """
        return self._zoom


class MarkFree(SCPICmdRead):
    """The ``MARK:FREE`` command.

    Description:
        - Returns how many marks are available for use. There can be a total of 1,024 marks
          returned.

    Usage:
        - Using the ``.query()`` method will send the ``MARK:FREE?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK:FREE?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - MARK:FREE?
        ```
    """


class MarkDelete(SCPICmdWrite):
    """The ``MARK:DELEte`` command.

    Description:
        - This command deletes a mark on a particular waveform, all waveforms in a column, the
          selected mark, or all marks.

    Usage:
        - Using the ``.write(value)`` method will send the ``MARK:DELEte value`` command.

    SCPI Syntax:
        ```
        - MARK:DELEte {CH<x>|MATH<x>|SELECTED|REF<x>|ALL|COLUMN}
        ```

    Info:
        - ``CH<x>`` deletes the mark on a channel waveform, where <x> is the channel number and can
          be 1, 2, 3, or 4.
        - ``MATH<x>`` deletes the mark on the math waveform, where <x> is the channel number and can
          be 1, 2, 3, or 4.
        - ``SELECTED`` deletes the mark on the selected waveform.
        - ``REF<x>`` deletes the mark on a reference waveform, where <x> is the reference waveform
          number and can be 1, 2, 3, or 4.
        - ``ALL`` deletes all marks on all waveforms.
        - ``COLUMN`` deletes marks on all waveforms in the current zoom pixel column.
    """


class MarkCreate(SCPICmdWrite):
    """The ``MARK:CREATE`` command.

    Description:
        - This command creates a mark on a specified waveform or all waveforms in a column.

    Usage:
        - Using the ``.write(value)`` method will send the ``MARK:CREATE value`` command.

    SCPI Syntax:
        ```
        - MARK:CREATE {CH<x>|MATH<x>|REF<x>|COLUMN}
        ```

    Info:
        - ``CH<x>`` creates the mark on a channel waveform, where <x> is the channel number and can
          be 1, 2, 3, or 4.
        - ``MATH<x>`` creates the mark on the math waveform, where <x> is the channel number and can
          be 1, 2, 3, or 4.
        - ``REF<x>`` creates the mark on a reference waveform, where <x> is the reference waveform
          number can be 1, 2, 3, or 4.
        - ``COLUMN`` creates marks on all waveforms in the current zoom pixel column.
    """


class Mark(SCPICmdWrite, SCPICmdRead):
    """The ``MARK`` command.

    Description:
        - Moves to the next or previous reference mark on the waveform. Returns the current mark
          position.

    Usage:
        - Using the ``.query()`` method will send the ``MARK?`` query.
        - Using the ``.verify(value)`` method will send the ``MARK?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``MARK value`` command.

    SCPI Syntax:
        ```
        - MARK {NEXT|PREVious}
        - MARK?
        ```

    Info:
        - ``NEXT`` moves to the next reference mark on the right.
        - ``PREVious`` moves to the next reference mark on the left.

    Properties:
        - ``.create``: The ``MARK:CREATE`` command.
        - ``.delete``: The ``MARK:DELEte`` command.
        - ``.free``: The ``MARK:FREE`` command.
        - ``.selected``: The ``MARK:SELECTED`` command tree.
        - ``.total``: The ``MARK:TOTal`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "MARK") -> None:
        super().__init__(device, cmd_syntax)
        self._create = MarkCreate(device, f"{self._cmd_syntax}:CREATE")
        self._delete = MarkDelete(device, f"{self._cmd_syntax}:DELEte")
        self._free = MarkFree(device, f"{self._cmd_syntax}:FREE")
        self._selected = MarkSelected(device, f"{self._cmd_syntax}:SELECTED")
        self._total = MarkTotal(device, f"{self._cmd_syntax}:TOTal")

    @property
    def create(self) -> MarkCreate:
        """Return the ``MARK:CREATE`` command.

        Description:
            - This command creates a mark on a specified waveform or all waveforms in a column.

        Usage:
            - Using the ``.write(value)`` method will send the ``MARK:CREATE value`` command.

        SCPI Syntax:
            ```
            - MARK:CREATE {CH<x>|MATH<x>|REF<x>|COLUMN}
            ```

        Info:
            - ``CH<x>`` creates the mark on a channel waveform, where <x> is the channel number and
              can be 1, 2, 3, or 4.
            - ``MATH<x>`` creates the mark on the math waveform, where <x> is the channel number and
              can be 1, 2, 3, or 4.
            - ``REF<x>`` creates the mark on a reference waveform, where <x> is the reference
              waveform number can be 1, 2, 3, or 4.
            - ``COLUMN`` creates marks on all waveforms in the current zoom pixel column.
        """
        return self._create

    @property
    def delete(self) -> MarkDelete:
        """Return the ``MARK:DELEte`` command.

        Description:
            - This command deletes a mark on a particular waveform, all waveforms in a column, the
              selected mark, or all marks.

        Usage:
            - Using the ``.write(value)`` method will send the ``MARK:DELEte value`` command.

        SCPI Syntax:
            ```
            - MARK:DELEte {CH<x>|MATH<x>|SELECTED|REF<x>|ALL|COLUMN}
            ```

        Info:
            - ``CH<x>`` deletes the mark on a channel waveform, where <x> is the channel number and
              can be 1, 2, 3, or 4.
            - ``MATH<x>`` deletes the mark on the math waveform, where <x> is the channel number and
              can be 1, 2, 3, or 4.
            - ``SELECTED`` deletes the mark on the selected waveform.
            - ``REF<x>`` deletes the mark on a reference waveform, where <x> is the reference
              waveform number and can be 1, 2, 3, or 4.
            - ``ALL`` deletes all marks on all waveforms.
            - ``COLUMN`` deletes marks on all waveforms in the current zoom pixel column.
        """
        return self._delete

    @property
    def free(self) -> MarkFree:
        """Return the ``MARK:FREE`` command.

        Description:
            - Returns how many marks are available for use. There can be a total of 1,024 marks
              returned.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:FREE?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:FREE?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:FREE?
            ```
        """
        return self._free

    @property
    def selected(self) -> MarkSelected:
        """Return the ``MARK:SELECTED`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:SELECTED?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:SELECTED?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.end``: The ``MARK:SELECTED:END`` command.
            - ``.focus``: The ``MARK:SELECTED:FOCUS`` command.
            - ``.label``: The ``MARK:SELECTED:LABel`` command.
            - ``.marksincolumn``: The ``MARK:SELECTED:MARKSINCOLumn`` command.
            - ``.owner``: The ``MARK:SELECTED:OWNer`` command.
            - ``.source``: The ``MARK:SELECTED:SOUrce`` command.
            - ``.start``: The ``MARK:SELECTED:STARt`` command.
            - ``.state``: The ``MARK:SELECTED:STATE`` command.
            - ``.zoom``: The ``MARK:SELECTED:ZOOm`` command tree.
        """
        return self._selected

    @property
    def total(self) -> MarkTotal:
        """Return the ``MARK:TOTal`` command.

        Description:
            - Returns how many marks are currently in use. There can be a total of 1,024 marks
              returned.

        Usage:
            - Using the ``.query()`` method will send the ``MARK:TOTal?`` query.
            - Using the ``.verify(value)`` method will send the ``MARK:TOTal?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - MARK:TOTal?
            ```
        """
        return self._total



# pyright: reportConstantRedefinition=none
"""The usbtmc commands module.

These commands are used in the following models:
MP5103

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - usbtmc.access
    ```
"""

from typing import Optional, TYPE_CHECKING, Union

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class Usbtmc(BaseTSPCmd):
    """The ``usbtmc`` command tree.

    Properties and methods:
        - ``.access``: The ``usbtmc.access`` attribute.
    """

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "usbtmc") -> None:
        super().__init__(device, cmd_syntax)

    @property
    def access(self) -> str:
        """Access the ``usbtmc.access`` attribute.

        Description:
            - This command is used to enable or disable the mainframe rear panel USB port access.

        Usage:
            - Accessing this property will send the ``print(usbtmc.access)`` query.
            - Setting this property to a value will send the ``usbtmc.access = value`` command.

        TSP Syntax:
            ```
            - usbtmc.access = value
            - print(usbtmc.access)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".access"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.access)"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.access`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @access.setter
    def access(self, value: Union[str, float]) -> None:
        """Access the ``usbtmc.access`` attribute.

        Description:
            - This command is used to enable or disable the mainframe rear panel USB port access.

        Usage:
            - Accessing this property will send the ``print(usbtmc.access)`` query.
            - Setting this property to a value will send the ``usbtmc.access = value`` command.

        TSP Syntax:
            ```
            - usbtmc.access = value
            - print(usbtmc.access)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".access", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.access = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to access the ``{self._cmd_syntax}.access`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""Module containing helpers for the `tm_devices` package."""

import contextlib
import importlib.metadata
import json
import logging
import platform
import re
import shlex
import socket
import subprocess  # nosec
import time
import warnings

from enum import EnumMeta
from functools import cache
from typing import Any, Dict, Optional, Tuple, Type

import requests

from packaging.version import InvalidVersion, Version

from tm_devices.helpers.constants_and_dataclasses import (
    _externally_registered_usbtmc_model_id_lookup,  # pyright: ignore[reportPrivateUsage]
    _USB_MODEL_ID_STR_LOOKUP,  # pyright: ignore[reportPrivateUsage]
    ConnectionTypes,
    DeviceConfigEntry,
    USBTMCConfiguration,
)
from tm_devices.helpers.enums import CustomStrEnum, SupportedModels
from tm_devices.helpers.standalone_helpers import (
    PACKAGE_NAME,
    PYVISA_PY_BACKEND,
    validate_address,
    VISA_RESOURCE_EXPRESSION_REGEX,
)

_logger: logging.Logger = logging.getLogger(__name__)

with warnings.catch_warnings():
    warnings.simplefilter("ignore", UserWarning)
    import pyvisa as visa

    from gpib_ctypes import make_default_gpib  # pyright: ignore[reportMissingTypeStubs]
    from pyvisa import util as pyvisa_util
    from pyvisa.resources import MessageBasedResource

    make_default_gpib()

####################################################################################################
# Private Constants
####################################################################################################
__SUPPORTED_MODEL_REGEX_STRING = (
    # AFGs
    rf"(?P<{SupportedModels.AFG3K.value}>^AFG3\d\d\d$)"
    rf"|(?P<{SupportedModels.AFG3KB.value}>^AFG3\d\d\dB$)"
    rf"|(?P<{SupportedModels.AFG3KC.value}>^AFG3\d\d\dC$)"
    rf"|(?P<{SupportedModels.AFG31K.value}>^AFG31\d\d\d$)"
    # AWGs
    rf"|(?P<{SupportedModels.AWG5200.value}>^AWG52\d\d$)"
    rf"|(?P<{SupportedModels.AWG5K.value}>^AWG5\d\d\d$)"
    rf"|(?P<{SupportedModels.AWG5KB.value}>^AWG5\d\d\dB$)"
    rf"|(?P<{SupportedModels.AWG5KC.value}>^AWG5\d\d\dC$)"
    rf"|(?P<{SupportedModels.AWG7K.value}>^AWG7\d\d\d$)"
    rf"|(?P<{SupportedModels.AWG7KB.value}>^AWG7\d\d\dB$)"
    rf"|(?P<{SupportedModels.AWG7KC.value}>^AWG7\d\d\dC$)"
    rf"|(?P<{SupportedModels.AWG70KA.value}>^AWG70\d\d\dA?$)"
    rf"|(?P<{SupportedModels.AWG70KB.value}>^AWG70\d\d\dB$)"
    # Scopes
    rf"|(?P<{SupportedModels.DPO5K.value}>^DPO5\d\d\d$)"
    rf"|(?P<{SupportedModels.DPO5KB.value}>^DPO5\d\d\dB$)"
    rf"|(?P<{SupportedModels.DPO7K.value}>^DPO7\d\d\d$)"
    rf"|(?P<{SupportedModels.DPO7KC.value}>^DPO7\d\d\dC$)"
    rf"|(?P<{SupportedModels.DPO70K.value}>^DPO7\d\d\d\d$)"
    rf"|(?P<{SupportedModels.DPO70KC.value}>^DPO7\d\d\d\dC$)"
    rf"|(?P<{SupportedModels.DPO70KD.value}>^DPO7\d\d\d\dD$)"
    rf"|(?P<{SupportedModels.DPO70KDX.value}>^DPO7\d\d\d\dDX$)"
    rf"|(?P<{SupportedModels.DPO70KSX.value}>^DPO7\d\d\d\dSX$)"
    rf"|(?P<{SupportedModels.DSA70K.value}>^DSA7\d\d\d\d$)"
    rf"|(?P<{SupportedModels.DSA70KC.value}>^DSA7\d\d\d\dC$)"
    rf"|(?P<{SupportedModels.DSA70KD.value}>^DSA7\d\d\d\dD$)"
    rf"|(?P<{SupportedModels.MSO2.value}>^MSO2\d$)"
    rf"|(?P<{SupportedModels.MSO4.value}>^MSO4\d$)"
    rf"|(?P<{SupportedModels.MSO4B.value}>^MSO4\dB$)"
    rf"|(?P<{SupportedModels.MSO5.value}>^MSO5\d$)"
    rf"|(?P<{SupportedModels.MSO5B.value}>^MSO5\dB$)"
    rf"|(?P<{SupportedModels.MSO5LP.value}>^MSO5\dLP$)"
    rf"|(?P<{SupportedModels.LPD6.value}>^LPD6\d$)"
    rf"|(?P<{SupportedModels.MSO6.value}>^MSO6\d$)"
    rf"|(?P<{SupportedModels.MSO6B.value}>^MSO6\dB$)"
    rf"|(?P<{SupportedModels.DPO7AX.value}>^DPO7\d\dAX$)"
    rf"|(?P<{SupportedModels.TEKSCOPEPC.value}>^TEKSCOPE(?:SW|PC)$)"
    rf"|(?P<{SupportedModels.MSO2K.value}>^MSO2\d\d\d$)"
    rf"|(?P<{SupportedModels.MSO2KB.value}>^MSO2\d\d\dB$)"
    rf"|(?P<{SupportedModels.DPO2K.value}>^DPO2\d\d\d$)"
    rf"|(?P<{SupportedModels.DPO2KB.value}>^DPO2\d\d\dB$)"
    rf"|(?P<{SupportedModels.MDO3.value}>^MDO3\d$)"
    rf"|(?P<{SupportedModels.MDO3K.value}>^MDO3\d\d\d$)"
    rf"|(?P<{SupportedModels.MSO4K.value}>^MSO4\d\d\d$)"
    rf"|(?P<{SupportedModels.MSO4KB.value}>^MSO4\d\d\dB$)"
    rf"|(?P<{SupportedModels.MDO4K.value}>^MDO4\d\d\d$)"
    rf"|(?P<{SupportedModels.MDO4KB.value}>^MDO4\d\d\dB$)"
    rf"|(?P<{SupportedModels.MDO4KC.value}>^MDO4\d\d\dC$)"
    rf"|(?P<{SupportedModels.DPO4K.value}>^DPO4\d\d\d$)"
    rf"|(?P<{SupportedModels.DPO4KB.value}>^DPO4\d\d\dB$)"
    rf"|(?P<{SupportedModels.MSO5K.value}>^MSO5\d\d\d$)"
    rf"|(?P<{SupportedModels.MSO5KB.value}>^MSO5\d\d\dB$)"
    rf"|(?P<{SupportedModels.MSO70K.value}>^MSO7\d\d\d\d$)"
    rf"|(?P<{SupportedModels.MSO70KC.value}>^MSO7\d\d\d\dC$)"
    rf"|(?P<{SupportedModels.MSO70KDX.value}>^MSO7\d\d\d\dDX$)"
    rf"|(?P<{SupportedModels.TSOVU.value}>^TSOVU$)"
    rf"|(?P<{SupportedModels.TMT4.value}>^TMT4$)"
    # SMUs
    rf"|(?P<{SupportedModels.SMU2400.value}>^2400$)"
    rf"|(?P<{SupportedModels.SMU2401.value}>^2401$)"
    rf"|(?P<{SupportedModels.SMU2410.value}>^2410$)"
    rf"|(?P<{SupportedModels.SMU2450.value}>^2450$)"
    rf"|(?P<{SupportedModels.SMU2460.value}>^2460$)"
    rf"|(?P<{SupportedModels.SMU2461.value}>^2461$)"
    rf"|(?P<{SupportedModels.SMU2470.value}>^2470$)"
    rf"|(?P<{SupportedModels.SMU2601B_PULSE.value}>^2601B-PULSE$)"
    rf"|(?P<{SupportedModels.SMU2601B.value}>^2601B$)"
    rf"|(?P<{SupportedModels.SMU2602B.value}>^2602B$)"
    rf"|(?P<{SupportedModels.SMU2604B.value}>^2604B$)"
    rf"|(?P<{SupportedModels.SMU2606B.value}>^2606B$)"
    rf"|(?P<{SupportedModels.SMU2611B.value}>^2611B$)"
    rf"|(?P<{SupportedModels.SMU2612B.value}>^2612B$)"
    rf"|(?P<{SupportedModels.SMU2614B.value}>^2614B$)"
    rf"|(?P<{SupportedModels.SMU2634B.value}>^2634B$)"
    rf"|(?P<{SupportedModels.SMU2635B.value}>^2635B$)"
    rf"|(?P<{SupportedModels.SMU2636B.value}>^2636B$)"
    rf"|(?P<{SupportedModels.SMU2651A.value}>^2651A$)"
    rf"|(?P<{SupportedModels.SMU2657A.value}>^2657A$)"
    rf"|(?P<{SupportedModels.SMU2601A.value}>^2601A$)"
    rf"|(?P<{SupportedModels.SMU2602A.value}>^2602A$)"
    rf"|(?P<{SupportedModels.SMU2604A.value}>^2604A$)"
    rf"|(?P<{SupportedModels.SMU2611A.value}>^2611A$)"
    rf"|(?P<{SupportedModels.SMU2612A.value}>^2612A$)"
    rf"|(?P<{SupportedModels.SMU2614A.value}>^2614A$)"
    rf"|(?P<{SupportedModels.SMU2634A.value}>^2634A$)"
    rf"|(?P<{SupportedModels.SMU2635A.value}>^2635A$)"
    rf"|(?P<{SupportedModels.SMU2636A.value}>^2636A$)"
    rf"|(?P<{SupportedModels.SMU6430.value}>^6430$)"
    rf"|(?P<{SupportedModels.SMU6514.value}>^6514$)"
    rf"|(?P<{SupportedModels.SMU6517B.value}>^6517B$)"
    # PSUs
    rf"|(?P<{SupportedModels.PSU2200.value}>^2200$)"
    rf"|(?P<{SupportedModels.PSU2220.value}>^2220$)"
    rf"|(?P<{SupportedModels.PSU2230.value}>^2230$)"
    rf"|(?P<{SupportedModels.PSU2231.value}>^2231$)"
    rf"|(?P<{SupportedModels.PSU2231A.value}>^2231A$)"
    rf"|(?P<{SupportedModels.PSU2280.value}>^2280$)"
    rf"|(?P<{SupportedModels.PSU2281.value}>^2281$)"
    # DAQs
    rf"|(?P<{SupportedModels.DAQ6510.value}>^DAQ6510$)"
    # DMMs
    rf"|(?P<{SupportedModels.DMM6500.value}>^DMM6500$)"
    rf"|(?P<{SupportedModels.DMM7510.value}>^DMM7510$)"
    rf"|(?P<{SupportedModels.DMM7512.value}>^DMM7512$)"
    # SSs
    rf"|(?P<{SupportedModels.SS3706A.value}>^3706A$)"
    # MFs
    rf"|(?P<{SupportedModels.MP5103.value}>^MP5103$)"
)
# NOTE: This regex would show as having a bunch of errors in the PyCharm IDE due to an
# open bug affecting f-strings in regex: https://youtrack.jetbrains.com/issue/PY-52140.
# For this reason, the regex string itself is stored in a separate, private constant.
_SUPPORTED_MODEL_REGEX_MAPPING = re.compile(__SUPPORTED_MODEL_REGEX_STRING)

_VALID_VERSION_REGEX = re.compile(r"^\d+\.\d+(\.\d+)*$")


####################################################################################################
# Public Functions
####################################################################################################
def check_for_update(package_name: str = PACKAGE_NAME, index_name: str = "pypi") -> None:
    """Check for an update for the provided package.

    Args:
        package_name: Check for an update for the provided package.
        index_name: The name of the index (pypi|test.pypi) to check for the package.
    """
    try:
        # Get the version from the local installation
        installed_version = importlib.metadata.version(package_name)

        # Get the version from the index
        url = f"https://{index_name}.org/pypi/{package_name}/json"
        response = requests.get(url, timeout=10)
        releases = json.loads(response.text)["releases"]
        version_list = sorted(releases, key=Version, reverse=True)
        latest_version = version_list[0]

        if installed_version != latest_version:
            _logger.debug(
                "\n\n\033[91mVersion %(latest_version)s of "
                "%(package_name)s is available on %(index_name)s.org.\n"
                "Version %(installed_version)s of "
                "%(package_name)s is currently installed.\n\n"
                "To upgrade %(package_name)s run the following command: "
                "python -m pip install -U %(package_name)s\n\n\033[0m",
                {
                    "latest_version": latest_version,
                    "package_name": package_name,
                    "index_name": index_name,
                    "installed_version": installed_version,
                },
            )
    except ModuleNotFoundError:
        _logger.warning(
            "\n\n\033[91m%s is not installed, unable to check for updates.\n\n\033[0m",
            package_name,
        )
    except (IndexError, ValueError):
        _logger.warning(
            "\n\n\033[91m%s is not available on %s.org, unable to check for updates.\n\n\033[0m",
            package_name,
            index_name,
        )


def check_network_connection(device_name: str, ip_address: str) -> Tuple[bool, str]:
    """Check the network connection to the device using the external ping command.

    Args:
        device_name: The name of the device.
        ip_address: The ip address of the device.

    Returns:
        A boolean indicating if there is a network connection and
            a string with the result of the ping command.
    """
    _logger.debug("(%s) ping >> %s", device_name, ip_address)
    ping_result = ping_address(ip_address)
    _logger.debug(
        "Response from ping >>\n%s",
        "\n".join(["    " + x for x in ping_result.strip().split("\n")]),
    )

    return "ttl=" in ping_result.lower(), ping_result


def check_port_connection(
    device_name: str, ip_address: str, port: int, timeout_seconds: int = 5
) -> bool:
    """Check if the given port is open on the device.

    Args:
        device_name: The name of the device.
        ip_address: The ip address.
        port: The port to check.
        timeout_seconds: The number of seconds to use as the socket timeout.

    Returns:
        A boolean indicating if the port is open.
    """
    _logger.debug("(%s) >> checking if port %d is open on %s", device_name, port, ip_address)

    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as temp_socket:
        try:
            temp_socket.settimeout(timeout_seconds)
            temp_socket.connect((ip_address, port))
            temp_socket.shutdown(socket.SHUT_RDWR)
            port_open = True
        except (socket.error, socket.gaierror, socket.herror):
            port_open = False

    _logger.debug("(%s) port %d open = %s", device_name, port, port_open)
    return port_open


def check_visa_connection(
    config_entry: DeviceConfigEntry, visa_library: str, device_name: str
) -> bool:
    """Check if a VISA connection can be made to the device.

    Args:
        config_entry: The device listed in the config of the framework to check.
        visa_library: Indicates which visa library to use for the connection.
        device_name: The name of the device.

    Returns:
        a boolean indicating if the visa connection was successful
    """
    _logger.debug(
        "(%s) >> checking if a VISA connection can be made to %s",
        device_name,
        config_entry.get_visa_resource_expression(),
    )
    try:
        # Create new VISA connection
        temp_resource = create_visa_connection(
            config_entry,
            visa_library=visa_library,
        )
        temp_resource.close()
        visa_connected = True
    except ConnectionError:  # raised by the create_visa_connection() function
        visa_connected = False

    _logger.debug("(%s) VISA connected = %s", device_name, visa_connected)
    return visa_connected


def create_visa_connection(
    device_config_entry: DeviceConfigEntry,
    visa_library: str,
    *,
    retry_connection: bool = False,
    second_connection_attempt_delay: int = 60,
    verbose_connection_failure_logging: bool = True,
) -> MessageBasedResource:
    """Create a VISA resource.

    Args:
        device_config_entry: The device config entry.
        visa_library: A string containing the VISA library to use to create a ResourceManager.
        retry_connection: Boolean indicating if a second connection attempt should be made. If True,
            two attempts are made to establish a VISA connection, with a configurable delay in
            between each attempt.
        second_connection_attempt_delay: The number of seconds to wait in between the first and
            second connection attempts when `retry_connection=True`.
        verbose_connection_failure_logging: Indicates if the connections failures should be logged
            with an ERROR level. If False, the failures will be logged with a DEBUG level.

    Returns:
        A VISA resource that can be passed into the device driver.

    Raises:
        ConnectionError: The VISA connection couldn't be created to the device.
    """
    failure_logging_level = logging.ERROR if verbose_connection_failure_logging else logging.DEBUG
    # Set up the VISA resource that will be used for communicating with the VISA device
    resource_expression = device_config_entry.get_visa_resource_expression()
    try:
        # noinspection PyTypeChecker
        visa_object: MessageBasedResource = visa.ResourceManager(  # pyright: ignore[reportAssignmentType]
            visa_library
        ).open_resource(resource_expression)
        # Print a warning if PyVISA-py is used when the user didn't specify STANDALONE
        if (
            visa_object.visalib.library_path.path == "py" and visa_library != PYVISA_PY_BACKEND
        ):  # pragma: no cover
            warning_msg = (
                "No VISA installation was detected, defaulting to using "
                "PyVISA-py as the VISA backend.\n\n"
                'Add the "STANDALONE" option to the configuration or set the '
                "`DeviceManager.visa_library` attribute to a valid VISA library to "
                "silence this warning."
            )
            warnings.warn(warning_msg, stacklevel=4)
            _logger.warning(warning_msg)
    # The broad except is because pyvisa_py can throw a base exception in the tcpip.py file
    except Exception as error_1:
        if not retry_connection:
            error_message = (
                f"Unable to establish a VISA connection to the "
                f"{device_config_entry.device_type.value}"
                f"{(' with the alias ' + repr(device_config_entry.alias)) if device_config_entry.alias else ''}"  # noqa: E501
                f" using the resource expression '{resource_expression}'"
                f" and the {repr(visa_library) if visa_library else 'default'} VISA library"
            )
            _logger.log(failure_logging_level, error_message)
            _logger.log(
                failure_logging_level,
                "1st exception: %s.%s: %s",
                error_1.__class__.__module__,
                error_1.__class__.__qualname__,
                error_1,
            )
            raise ConnectionError(error_message) from error_1
        _logger.debug(
            "Initial connection attempt failed, waiting %d second(s) "
            "before re-attempting to create the VISA connection",
            second_connection_attempt_delay,
        )
        time.sleep(second_connection_attempt_delay)
        try:
            # noinspection PyTypeChecker
            visa_object: MessageBasedResource = visa.ResourceManager(  # pyright: ignore[reportAssignmentType]
                visa_library
            ).open_resource(resource_expression)
        # The broad except is because pyvisa_py can throw a base exception in the tcpip.py file
        except Exception as error_2:
            error_message = (
                f"Unable to establish a VISA connection (after two tries) to the "
                f"{device_config_entry.device_type.value}"
                f"{(' with the alias ' + repr(device_config_entry.alias)) if device_config_entry.alias else ''}"  # noqa: E501
                f" using the resource expression '{resource_expression}'"
                f" and the {repr(visa_library) if visa_library else 'default'} VISA library"
            )
            _logger.log(failure_logging_level, error_message)
            _logger.log(
                failure_logging_level,
                "1st exception: %s.%s: %s",
                error_1.__class__.__module__,
                error_1.__class__.__qualname__,
                error_1,
            )
            _logger.log(
                failure_logging_level,
                "2nd exception: %s.%s: %s:",
                error_2.__class__.__module__,
                error_2.__class__.__qualname__,
                error_2,
            )
            ping_message = ""
            ping_output = ""
            if device_config_entry.connection_type in {
                ConnectionTypes.TCPIP,
                ConnectionTypes.SOCKET,
            }:
                ping_output = (
                    ping_address(device_config_entry.address)
                    or "\nno response returned or unable to find device\n"
                )
                ping_message = (
                    f"\n\nThis is the current ping output for the device at "
                    f"{device_config_entry.address}:\n"
                )
                _logger.debug("%s%s", ping_message.lstrip(), ping_output)
            raise ConnectionError(error_message + ping_message + ping_output) from error_2

    return _configure_visa_object(visa_object, device_config_entry, visa_library)


def detect_visa_resource_expression(input_str: str) -> Optional[Tuple[str, str]]:
    """Check if a given string is a VISA resource expression.

    This function will check if a string is a VISA resource expression and pull out the pieces
    needed to make a connection using the DeviceManager.

    The pieces consist of:
        - The connection type, e.g. TCPIP, GPIB<board_number> (e.g. GPIB0).
        - The address of the device, an IP address
          (with port separated by a colon for SOCKET connections), hostname, or
          string in the format ``model-serial``.

    Args:
        input_str: The string to check for a VISA resource expression.

    Returns:
        A tuple with the connection information parts.
    """
    retval: Optional[Tuple[str, str]] = None
    if input_str.upper().startswith("ASRL"):
        retval = (ConnectionTypes.SERIAL.value, input_str[4:].split("::", 1)[0])
    elif (match := VISA_RESOURCE_EXPRESSION_REGEX.search(input_str.upper())) is not None:
        match_groups_list = list(filter(None, match.groups()))
        for unneeded_part in ("INST", "INST0", "INSTR"):
            while unneeded_part in match_groups_list:
                match_groups_list.remove(unneeded_part)
        # Check if the model is in the USB model lookup
        model_id_lookup = {
            **_externally_registered_usbtmc_model_id_lookup,
            **_USB_MODEL_ID_STR_LOOKUP,
        }
        filtered_usb_model_keys = [
            key
            for key, value in model_id_lookup.items()
            # Model id also has an alpha character which needs to be converted to lowercase
            if value.model_id.lower() == match_groups_list[1].lower()
        ]
        if filtered_usb_model_keys:
            # SMU and PSU need to be removed from the string to prevent issues
            match_groups_list[1] = filtered_usb_model_keys[0].replace("SMU", "").replace("PSU", "")
        if match_groups_list[-1] == ConnectionTypes.SOCKET.value:
            retval = (match_groups_list[-1], ":".join(match_groups_list[1:3]))
        # If connection_type is GPIB, the board number must be passed back in the returned value
        elif input_str.upper().startswith(ConnectionTypes.GPIB.value.upper()):
            retval = (
                match_groups_list[0],
                "-".join(match_groups_list[1:]).lstrip("0X"),
            )
        else:
            retval = (
                match_groups_list[0].rstrip("0"),
                "-".join(match_groups_list[1:]).lstrip("0X"),
            )
    return retval


def get_model_series(model: str) -> str:
    """Get the series string from the full model number.

    Args:
        model: The full model string (ex. MSO58, LPD64).

    Returns:
        The model series string (ex. MSO5, LPD6).
    """
    model_parts = model.strip().upper().split("-")
    simplified_model = model_parts[0].replace("MODEL", "").strip()

    # Remove "Virtual" from the model string
    simplified_model = simplified_model.replace("VIRTUAL", "")

    # Remove ending characters from the model string that doesn't
    # contribute to determining the correct series.
    valid_model_endings = {"A", "B", "C", "D", "LP"}
    model_end = ""
    if re.search(r"\d", simplified_model):  # if the model contains numbers
        model_end = re.split(r"\d+", simplified_model)[-1]  # split on the occurrence of each number
    if len(model_end) == 1 and model_end not in valid_model_endings:
        simplified_model = simplified_model.rstrip(model_end)

    # Check for any postscripts, e.g. 2601B-Pulse where "Pulse" is the postscript, which are a
    # necessary part of determining the correct series and add them back to the model string
    # to use to determine the series.
    if len(model_parts) > 1:
        for count, part in enumerate(model_parts):
            # avoid first model part as a postscript will likely never be in the beginning
            if count and len(part) > 1 and all(x.isalpha() for x in part):
                simplified_model += f"-{part}"

    # Find the series by checking against the regex mapping
    model_series = ""
    if match := _SUPPORTED_MODEL_REGEX_MAPPING.match(simplified_model):
        match_dict = match.groupdict()
        filtered_dict = dict(filter(lambda item: item[1] is not None, match_dict.items()))
        with contextlib.suppress(StopIteration):
            model_series = next(iter(filtered_dict.keys()))

    # Warn the user if the model is not in the regex mapping or list of supported models,
    # and therefore not officially supported.
    if not model_series:
        if model not in SupportedModels.list_values():
            warnings.warn(f'The "{model}" model is not supported by {PACKAGE_NAME}', stacklevel=2)
            _logger.warning('The "%s" model is not supported by %s', model, PACKAGE_NAME)
        model_series = model
    return model_series


def get_version(version_string: str) -> Version:
    """Get a Version object from a string.

    Args:
        version_string: The string containing the version to create.

    Returns:
        The Version object.
    """
    version_parts = version_string.split(".")
    found_alpha = False
    try:
        version = Version(version_string)
    except InvalidVersion:
        if all(x.isalpha() for x in version_parts[-1]) and all(
            y.isdigit() for y in version_parts[:-1]
        ):
            version = Version(
                version_string.replace("." + version_parts[-1], "+" + version_parts[-1])
            )
        elif version_string.count("-") == 1 and _VALID_VERSION_REGEX.match(
            version_string.split("-")[0]
        ):
            new_version_string = version_string.replace("-", "+", 1)
            if new_version_string.count("+") == 2:  # noqa: PLR2004
                new_version_string = "-".join(new_version_string.rsplit("+", 1))
            version = Version(new_version_string)
        else:
            output_str = ""
            for char in version_parts[-1]:
                if char.isalpha() and not found_alpha:
                    output_str += "+" + char
                    found_alpha = True
                else:
                    output_str += char
            version = Version(version_string.replace(version_parts[-1], output_str))
    return version


def get_visa_backend(visa_lib_path: str) -> str:
    """Create a human-readable VISA backend name based on the currently used VISA library path.

    Args:
        visa_lib_path: The path to the currently used VISA library.

    Returns:
        A string indicating the current VISA backend.
    """
    visa_name = ""

    system_visa_info = _get_system_visa_info()
    # noinspection PyTypeChecker
    visa_backends: Dict[str, Any] = system_visa_info["backends"]

    # Figure out which visa is being used
    try:
        if visa_lib_path == "py":
            visa_name = "PyVISA-py"
        else:
            raise KeyError  # noqa: TRY301
    except KeyError:
        found_visa = False
        for visa_type, visa_implementation_dict in visa_backends.items():
            for (
                visa_implementation_name,
                visa_implementation_details,
            ) in visa_implementation_dict.items():
                if visa_lib_path in visa_implementation_name:
                    vendor = visa_implementation_details["Vendor"]
                    visa_name = "NI-VISA" if "National Instruments" in vendor else f"{vendor} VISA"
                    found_visa = True
                elif visa_lib_path.endswith("yaml") and visa_type == "sim":
                    visa_name = "PyVISA-sim"
                    found_visa = True
            if found_visa:
                break

    return visa_name


def ping_address(address: str, timeout: int = 2) -> str:
    """Ping the given address.

    Args:
        address: The address to ping, either an IP or hostname.
        timeout: The timeout to use, in seconds.

    Returns:
         The response of the ping command.
    """
    ping_command = _create_ping_command(address, timeout)

    try:
        output = subprocess.check_output(shlex.split(ping_command)).decode("utf-8")  # noqa: S603
    except subprocess.CalledProcessError:
        output = ""
    return output.replace("\r\n", "\n")


def register_additional_usbtmc_mapping(model_series: str, *, model_id: str, vendor_id: str) -> None:
    """Register USBTMC connection information for a device that doesn't have native `tm_devices` USBTMC support.

    This function adds an additional mapping between the given ``model_series`` and the USBTMC
    connection information provided. This mapping can then be used by ``tm_devices`` to create a
    USBTMC connection to the device.

    Args:
        model_series: The model series string. For VISA devices, the model series is based on the
            model that is returned from the ``*IDN?`` query (See the
            [``get_model_series()``][tm_devices.helpers.get_model_series] function for details). For
            REST API devices, the model series is provided via the
            [``device_driver``][tm_devices.helpers.constants_and_dataclasses.DeviceConfigEntry.device_driver]
            parameter in the configuration file, environment variable, or Python code.
        model_id: The hexadecimal Model ID used to create the USBTMC resource expression.
        vendor_id: The hexadecimal Vendor ID used to create the USBTMC resource expression.
    """  # noqa: E501
    _externally_registered_usbtmc_model_id_lookup[model_series] = USBTMCConfiguration(
        model_id=model_id, vendor_id=vendor_id
    )


def sanitize_enum(value: str, enum_class: Type[CustomStrEnum]) -> CustomStrEnum:
    """Sanitize a string value into its enum value.

    Args:
        value: The value to check for.
        enum_class: The Enum class to look in.

    Returns:
        The enum value corresponding to the string value passed in.

    Raises:
        TypeError: Indicates that the value doesn't exist in the enum
    """
    if string_in_enum(value, enum_class):
        sanitized_enum = enum_class[value]
    else:
        msg = (
            f"{value} is not a valid enum of {enum_class.__name__}. "
            f"Valid options are: {enum_class.list_values()}"
        )
        raise TypeError(msg)

    return sanitized_enum


def string_in_enum(value: str, enum_class: EnumMeta) -> bool:
    """Check if a string value exists in an Enum class object.

    Args:
        value: The value to check for.
        enum_class: The Enum class to look in.

    Returns:
        A boolean indicating if the string exists in the Enum.
    """
    try:
        _ = enum_class[value]
    except KeyError:
        return False
    return True


####################################################################################################
# Private Functions
####################################################################################################
def _create_ping_command(address: str, timeout: int = 2) -> str:
    """Create a ping command to ping the given address.

    Args:
        address: The address to ping, either an IP or hostname.
        timeout: The timeout to use, in seconds.

    Returns:
         The response of the ping command.
    """
    validate_address(address)
    # Create the ping command
    if (platform_system := platform.system().lower()) == "windows":
        count_arg = "-n"
        timeout_arg = "-w"
        timeout_val = timeout * 1000  # Windows uses milliseconds
    else:
        count_arg = "-c"
        timeout_arg = "-W" if platform_system == "darwin" else "-w"
        timeout_val = timeout  # Linux/MacOS uses seconds
    return f"ping {address} {count_arg} 1 {timeout_arg} {timeout_val}"


def _configure_visa_object(
    visa_object: MessageBasedResource,
    device_config_entry: DeviceConfigEntry,
    visa_library: str,
) -> MessageBasedResource:
    """Configure settings on the VISA resource object.

    Args:
        visa_object: The VISA resource object to configure.
        device_config_entry: The device config entry.
        visa_library: A string containing the VISA library to use to create a ResourceManager.

    Returns:
        A VISA resource that can be passed into the device driver.
    """
    visa_object.encoding = "utf-8"

    if visa_library.endswith("@sim"):  # pragma: no cover
        visa_object.read_termination = "\n"
        visa_object.write_termination = "\n"

    if isinstance(visa_object, visa.resources.TCPIPSocket):
        # Socket connections seem to need the termination characters specified.
        visa_object.write_termination = "\n"
        visa_object.read_termination = "\n"

        if visa_library != PYVISA_PY_BACKEND:  # pragma: no cover
            visa_object.send_end = True
    elif (
        isinstance(visa_object, visa.resources.SerialInstrument)
        and device_config_entry.serial_config
    ):
        serial_config = device_config_entry.serial_config
        # update serial fields if they were specified, otherwise use the serial visa object defaults
        for name in serial_config.to_dict():
            if (value := getattr(serial_config, name)) is not None:
                setattr(visa_object, name, value)
            else:
                setattr(serial_config, name, getattr(visa_object, name))

    return visa_object


@cache
def _get_system_visa_info() -> Dict[str, Any]:
    """Get the VISA information for the current system.

    Returns:
        A dictionary with the VISA info for the system.
    """
    fetch_backend_info = True

    if platform.system().lower() == "darwin":
        try:
            output = subprocess.check_output(shlex.split("csrutil status")).decode(  # noqa: S603
                "utf-8"
            )
        except subprocess.SubprocessError:
            output = ""
        if "System Integrity Protection status: enabled" in output:
            fetch_backend_info = False

    return pyvisa_util.get_system_details(backends=fetch_backend_info)



"""The newpass commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7AX, DPO7K,
DPO7KC, DSA70KC, DSA70KD, LPD6, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2, MSO2K, MSO2KB, MSO4,
MSO4B, MSO4K, MSO4KB, MSO5, MSO5B, MSO5K, MSO5KB, MSO5LP, MSO6, MSO6B, MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - NEWpass <QString>
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Newpass(SCPICmdWrite):
    """The ``NEWpass`` command.

    Description:
        - This command (no query form) changes the password that enables access to password
          protected data. The PASSWord command must be successfully executed before using this
          command or an execution error will be generated.

    Usage:
        - Using the ``.write(value)`` method will send the ``NEWpass value`` command.

    SCPI Syntax:
        ```
        - NEWpass <QString>
        ```

    Info:
        - ``<QString>`` is the new password, which can contain up to 10 characters.
    """

    _WRAP_ARG_WITH_QUOTES = True

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "NEWpass") -> None:
        super().__init__(device, cmd_syntax)



"""The wavfrm commands module.

These commands are used in the following models:
DPO2K, DPO2KB, DPO4K, DPO4KB, DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7AX, DPO7K,
DPO7KC, DSA70KC, DSA70KD, LPD6, MDO3, MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2, MSO2K, MSO2KB, MSO4,
MSO4B, MSO4K, MSO4KB, MSO5, MSO5B, MSO5K, MSO5KB, MSO5LP, MSO6, MSO6B, MSO70KC, MSO70KDX, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - WAVFrm?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Wavfrm(SCPICmdRead):
    """The ``WAVFrm`` command.

    Description:
        - This query-only command provides the Tektronix standard waveform query which returns the
          waveform preamble followed by the waveform data for the source specified by
          ``:DATa:SOUrce`` using the ``:DATa`` settings for encoding, width, and so forth.

    Usage:
        - Using the ``.query()`` method will send the ``WAVFrm?`` query.
        - Using the ``.verify(value)`` method will send the ``WAVFrm?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - WAVFrm?
        ```
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "WAVFrm") -> None:
        super().__init__(device, cmd_syntax)



"""The connectivity commands module.

These commands are used in the following models:
AWG5200, AWG70KA, AWG70KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CONNectivity:ACTive <generator_name>
    - CONNectivity:ACTive?
    - CONNectivity:CONNect <hostname>[,<generator_name>]
    - CONNectivity:DISConnect <generator_name>
    - CONNectivity:GANG:CREAte <gang_name>,<member1>,<member2>[,<member3>[,<member4>]]
    - CONNectivity:REMove <generator_name>
    - CONNectivity:STATus? <generator_name>
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdReadWithArguments, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class ConnectivityStatus(SCPICmdReadWithArguments):
    """The ``CONNectivity:STATus`` command.

    Description:
        - This command returns the connection status of the named generator.

    Usage:
        - Using the ``.query(argument)`` method will send the ``CONNectivity:STATus? argument``
          query.
        - Using the ``.verify(argument, value)`` method will send the
          ``CONNectivity:STATus? argument`` query and raise an AssertionError if the returned value
          does not match ``value``.

    SCPI Syntax:
        ```
        - CONNectivity:STATus? <generator_name>
        ```
    """


class ConnectivityRemove(SCPICmdWrite):
    """The ``CONNectivity:REMove`` command.

    Description:
        - This command removes the named generator from SourceXpress. If the named generator is the
          active generator, it is disconnected and removed without warning. If the generator is in a
          gang, the gang and all its members are removed.

    Usage:
        - Using the ``.write(value)`` method will send the ``CONNectivity:REMove value`` command.

    SCPI Syntax:
        ```
        - CONNectivity:REMove <generator_name>
        ```
    """


class ConnectivityGangCreate(SCPICmdWrite):
    """The ``CONNectivity:GANG:CREAte`` command.

    Description:
        - This command creates a gang generator consisting of the specified instruments (members). A
          gang generator can consist of 2 to 4 generators. The gang is placed in the Generator List.

    Usage:
        - Using the ``.write(value)`` method will send the ``CONNectivity:GANG:CREAte value``
          command.

    SCPI Syntax:
        ```
        - CONNectivity:GANG:CREAte <gang_name>,<member1>,<member2>[,<member3>[,<member4>]]
        ```
    """


class ConnectivityGang(SCPICmdRead):
    """The ``CONNectivity:GANG`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``CONNectivity:GANG?`` query.
        - Using the ``.verify(value)`` method will send the ``CONNectivity:GANG?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.create``: The ``CONNectivity:GANG:CREAte`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._create = ConnectivityGangCreate(device, f"{self._cmd_syntax}:CREAte")

    @property
    def create(self) -> ConnectivityGangCreate:
        """Return the ``CONNectivity:GANG:CREAte`` command.

        Description:
            - This command creates a gang generator consisting of the specified instruments
              (members). A gang generator can consist of 2 to 4 generators. The gang is placed in
              the Generator List.

        Usage:
            - Using the ``.write(value)`` method will send the ``CONNectivity:GANG:CREAte value``
              command.

        SCPI Syntax:
            ```
            - CONNectivity:GANG:CREAte <gang_name>,<member1>,<member2>[,<member3>[,<member4>]]
            ```
        """
        return self._create


class ConnectivityDisconnect(SCPICmdWrite):
    """The ``CONNectivity:DISConnect`` command.

    Description:
        - This command disconnects the named generator from SourceXpress.

    Usage:
        - Using the ``.write(value)`` method will send the ``CONNectivity:DISConnect value``
          command.

    SCPI Syntax:
        ```
        - CONNectivity:DISConnect <generator_name>
        ```
    """


class ConnectivityConnect(SCPICmdWrite):
    """The ``CONNectivity:CONNect`` command.

    Description:
        - This command connects SourceXpress to a remote generator using the generator's hostname,
          making the remote generator available for use by SourceXpress. You can also assign a
          unique generator name of the connected generator. If a name is not specified, then the
          hostname is used as the name for the connected generator. The generator name is the name
          that appears in the Generator List.

    Usage:
        - Using the ``.write(value)`` method will send the ``CONNectivity:CONNect value`` command.

    SCPI Syntax:
        ```
        - CONNectivity:CONNect <hostname>[,<generator_name>]
        ```
    """


class ConnectivityActive(SCPICmdWrite, SCPICmdRead):
    """The ``CONNectivity:ACTive`` command.

    Description:
        - This command sets or returns the active generator using the connected generator name. The
          active generator is the target of instrument operations. Only a single generator can be
          active at once. The set version of this command must use the actual generator name as
          listed in the Generator List (in the SourceXpress application).

    Usage:
        - Using the ``.query()`` method will send the ``CONNectivity:ACTive?`` query.
        - Using the ``.verify(value)`` method will send the ``CONNectivity:ACTive?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CONNectivity:ACTive value`` command.

    SCPI Syntax:
        ```
        - CONNectivity:ACTive <generator_name>
        - CONNectivity:ACTive?
        ```
    """


class Connectivity(SCPICmdRead):
    """The ``CONNectivity`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``CONNectivity?`` query.
        - Using the ``.verify(value)`` method will send the ``CONNectivity?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.active``: The ``CONNectivity:ACTive`` command.
        - ``.connect``: The ``CONNectivity:CONNect`` command.
        - ``.disconnect``: The ``CONNectivity:DISConnect`` command.
        - ``.gang``: The ``CONNectivity:GANG`` command tree.
        - ``.remove``: The ``CONNectivity:REMove`` command.
        - ``.status``: The ``CONNectivity:STATus`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "CONNectivity"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._active = ConnectivityActive(device, f"{self._cmd_syntax}:ACTive")
        self._connect = ConnectivityConnect(device, f"{self._cmd_syntax}:CONNect")
        self._disconnect = ConnectivityDisconnect(device, f"{self._cmd_syntax}:DISConnect")
        self._gang = ConnectivityGang(device, f"{self._cmd_syntax}:GANG")
        self._remove = ConnectivityRemove(device, f"{self._cmd_syntax}:REMove")
        self._status = ConnectivityStatus(device, f"{self._cmd_syntax}:STATus")

    @property
    def active(self) -> ConnectivityActive:
        """Return the ``CONNectivity:ACTive`` command.

        Description:
            - This command sets or returns the active generator using the connected generator name.
              The active generator is the target of instrument operations. Only a single generator
              can be active at once. The set version of this command must use the actual generator
              name as listed in the Generator List (in the SourceXpress application).

        Usage:
            - Using the ``.query()`` method will send the ``CONNectivity:ACTive?`` query.
            - Using the ``.verify(value)`` method will send the ``CONNectivity:ACTive?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CONNectivity:ACTive value``
              command.

        SCPI Syntax:
            ```
            - CONNectivity:ACTive <generator_name>
            - CONNectivity:ACTive?
            ```
        """
        return self._active

    @property
    def connect(self) -> ConnectivityConnect:
        """Return the ``CONNectivity:CONNect`` command.

        Description:
            - This command connects SourceXpress to a remote generator using the generator's
              hostname, making the remote generator available for use by SourceXpress. You can also
              assign a unique generator name of the connected generator. If a name is not specified,
              then the hostname is used as the name for the connected generator. The generator name
              is the name that appears in the Generator List.

        Usage:
            - Using the ``.write(value)`` method will send the ``CONNectivity:CONNect value``
              command.

        SCPI Syntax:
            ```
            - CONNectivity:CONNect <hostname>[,<generator_name>]
            ```
        """
        return self._connect

    @property
    def disconnect(self) -> ConnectivityDisconnect:
        """Return the ``CONNectivity:DISConnect`` command.

        Description:
            - This command disconnects the named generator from SourceXpress.

        Usage:
            - Using the ``.write(value)`` method will send the ``CONNectivity:DISConnect value``
              command.

        SCPI Syntax:
            ```
            - CONNectivity:DISConnect <generator_name>
            ```
        """
        return self._disconnect

    @property
    def gang(self) -> ConnectivityGang:
        """Return the ``CONNectivity:GANG`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``CONNectivity:GANG?`` query.
            - Using the ``.verify(value)`` method will send the ``CONNectivity:GANG?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.create``: The ``CONNectivity:GANG:CREAte`` command.
        """
        return self._gang

    @property
    def remove(self) -> ConnectivityRemove:
        """Return the ``CONNectivity:REMove`` command.

        Description:
            - This command removes the named generator from SourceXpress. If the named generator is
              the active generator, it is disconnected and removed without warning. If the generator
              is in a gang, the gang and all its members are removed.

        Usage:
            - Using the ``.write(value)`` method will send the ``CONNectivity:REMove value``
              command.

        SCPI Syntax:
            ```
            - CONNectivity:REMove <generator_name>
            ```
        """
        return self._remove

    @property
    def status(self) -> ConnectivityStatus:
        """Return the ``CONNectivity:STATus`` command.

        Description:
            - This command returns the connection status of the named generator.

        Usage:
            - Using the ``.query(argument)`` method will send the ``CONNectivity:STATus? argument``
              query.
            - Using the ``.verify(argument, value)`` method will send the
              ``CONNectivity:STATus? argument`` query and raise an AssertionError if the returned
              value does not match ``value``.

        SCPI Syntax:
            ```
            - CONNectivity:STATus? <generator_name>
            ```
        """
        return self._status



"""The header commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - HEADer {ON|OFF|<NR1>}
    - HEADer?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Header(SCPICmdWrite, SCPICmdRead):
    """The ``HEADer`` command.

    Description:
        - This command sets or queries the Response Header Enable State that causes the instrument
          to either include or omit headers on query responses. Whether the long or short form of
          header keywords and enumerations are returned is dependent upon the state of ``:VERBose``.
          Note: This command does not affect IEEE Std 488.2-1987 Common Commands (those starting
          with an asterisk); these commands never return headers.

    Usage:
        - Using the ``.query()`` method will send the ``HEADer?`` query.
        - Using the ``.verify(value)`` method will send the ``HEADer?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HEADer value`` command.

    SCPI Syntax:
        ```
        - HEADer {ON|OFF|<NR1>}
        - HEADer?
        ```

    Info:
        - ``<NR1>`` = 0 sets the Response Header Enable State to false; any other value sets this
          state to true.
        - ``OFF`` sets the Response Header Enable State to false. This causes the instrument to omit
          headers on query responses, so that only the argument is returned.
        - ``ON`` sets the Response Header Enable State to true. This causes the instrument to
          include headers on applicable query responses. You can then use the query response as a
          command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "HEADer") -> None:
        super().__init__(device, cmd_syntax)



"""The horizontal commands module.

These commands are used in the following models:
TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - HORizontal:DELay:MODe {ON|OFF|<NR1>}
    - HORizontal:DELay:MODe?
    - HORizontal:DELay:TIMe <NR3>
    - HORizontal:DELay:TIMe?
    - HORizontal:MODe {AUTO|MANual}
    - HORizontal:MODe:MANual:CONFIGure {HORIZontalscale|RECORDLength}
    - HORizontal:MODe:MANual:CONFIGure?
    - HORizontal:MODe?
    - HORizontal:POSition <NR3>
    - HORizontal:POSition?
    - HORizontal:SAMPLERate <NR3>
    - HORizontal:SAMPLERate?
    - HORizontal:SCAle <NR3>
    - HORizontal:SCAle?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class HorizontalScale(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:SCAle`` command.

    Description:
        - This command sets or queries the horizontal scale.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:SCAle?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:SCAle?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:SCAle value`` command.

    SCPI Syntax:
        ```
        - HORizontal:SCAle <NR3>
        - HORizontal:SCAle?
        ```

    Info:
        - ``<NR3>`` is the horizontal scale in time per division.
    """


class HorizontalSamplerate(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:SAMPLERate`` command.

    Description:
        - This command sets or queries the horizontal sample rate.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:SAMPLERate?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:SAMPLERate?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:SAMPLERate value`` command.

    SCPI Syntax:
        ```
        - HORizontal:SAMPLERate <NR3>
        - HORizontal:SAMPLERate?
        ```

    Info:
        - ``<NR3>`` is the horizontal sample rate in samples per second.
    """


class HorizontalPosition(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:POSition`` command.

    Description:
        - This command sets or queries the horizontal position as a percent of screen width. When
          Horizontal Delay Mode is turned off, this command is equivalent to adjusting the
          HORIZONTAL POSITION knob on the front panel. When Horizontal Delay Mode is turned on, the
          horizontal position is forced to 50%.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:POSition?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:POSition?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:POSition value`` command.

    SCPI Syntax:
        ```
        - HORizontal:POSition <NR3>
        - HORizontal:POSition?
        ```

    Info:
        - ``<NR3>`` is from 0 to ≈100 and is the position of the trigger point on the screen (0 =
          left edge, 100 = right edge).
    """


class HorizontalModeManualConfigure(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:MODe:MANual:CONFIGure`` command.

    Description:
        - Sets or queries which horizontal control (scale or record length) will primarily change
          when the sample rate is changed in Manual mode. If the selected control (scale or record
          length) reaches a limit then the unselected control (record length or scale) may also
          change.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:MODe:MANual:CONFIGure?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:MODe:MANual:CONFIGure?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``HORizontal:MODe:MANual:CONFIGure value`` command.

    SCPI Syntax:
        ```
        - HORizontal:MODe:MANual:CONFIGure {HORIZontalscale|RECORDLength}
        - HORizontal:MODe:MANual:CONFIGure?
        ```

    Info:
        - ``HORIZontalscale`` will change when sample rate is adjusted.
        - ``RECORDLength`` will change when sample rate is adjusted.
    """


class HorizontalModeManual(SCPICmdRead):
    """The ``HORizontal:MODe:MANual`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:MODe:MANual?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:MODe:MANual?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.configure``: The ``HORizontal:MODe:MANual:CONFIGure`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._configure = HorizontalModeManualConfigure(device, f"{self._cmd_syntax}:CONFIGure")

    @property
    def configure(self) -> HorizontalModeManualConfigure:
        """Return the ``HORizontal:MODe:MANual:CONFIGure`` command.

        Description:
            - Sets or queries which horizontal control (scale or record length) will primarily
              change when the sample rate is changed in Manual mode. If the selected control (scale
              or record length) reaches a limit then the unselected control (record length or scale)
              may also change.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:MODe:MANual:CONFIGure?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``HORizontal:MODe:MANual:CONFIGure?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``HORizontal:MODe:MANual:CONFIGure value`` command.

        SCPI Syntax:
            ```
            - HORizontal:MODe:MANual:CONFIGure {HORIZontalscale|RECORDLength}
            - HORizontal:MODe:MANual:CONFIGure?
            ```

        Info:
            - ``HORIZontalscale`` will change when sample rate is adjusted.
            - ``RECORDLength`` will change when sample rate is adjusted.
        """
        return self._configure


class HorizontalMode(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:MODe`` command.

    Description:
        - This command set or queries the horizontal operating mode.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:MODe?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:MODe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:MODe value`` command.

    SCPI Syntax:
        ```
        - HORizontal:MODe {AUTO|MANual}
        - HORizontal:MODe?
        ```

    Info:
        - ``AUTO`` selects the automatic horizontal model. Auto mode automatically adjusts the
          sample rate and record length to provide a high acquisition rate in Fast Acq or signal
          fidelity in analysis. Record length is read only.
        - ``MANUAL`` selects the manual horizontal model. Manual mode lets you change the sample
          rate, horizontal scale, and record length. These values interact. For example, when you
          change record length then the horizontal scale also changes.

    Properties:
        - ``.manual``: The ``HORizontal:MODe:MANual`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._manual = HorizontalModeManual(device, f"{self._cmd_syntax}:MANual")

    @property
    def manual(self) -> HorizontalModeManual:
        """Return the ``HORizontal:MODe:MANual`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:MODe:MANual?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:MODe:MANual?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.configure``: The ``HORizontal:MODe:MANual:CONFIGure`` command.
        """
        return self._manual


class HorizontalDelayTime(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:DELay:TIMe`` command.

    Description:
        - This command sets or queries the horizontal delay time that is used when delay mode is on.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:DELay:TIMe?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:DELay:TIMe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:DELay:TIMe value`` command.

    SCPI Syntax:
        ```
        - HORizontal:DELay:TIMe <NR3>
        - HORizontal:DELay:TIMe?
        ```

    Info:
        - ``NR3`` is the delay in seconds.
    """


class HorizontalDelayMode(SCPICmdWrite, SCPICmdRead):
    """The ``HORizontal:DELay:MODe`` command.

    Description:
        - This command sets or queries the horizontal delay mode.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:DELay:MODe?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:DELay:MODe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``HORizontal:DELay:MODe value`` command.

    SCPI Syntax:
        ```
        - HORizontal:DELay:MODe {ON|OFF|<NR1>}
        - HORizontal:DELay:MODe?
        ```

    Info:
        - ``OFF`` sets the Horizontal Delay Mode to off. This causes the ``HORizontal:POSition``
          command to operate like the HORIZONTAL POSITION knob on the front panel.
        - ``ON`` sets the Horizontal Delay Mode to on. This causes the ``HORizontal:DELay:TIMe``
          command to operate like the HORIZONTAL POSITION knob on the front panel.
        - ``<NR1>`` = 0 sets the Horizontal Delay Mode to off; any other value sets this mode to on.
    """


class HorizontalDelay(SCPICmdRead):
    """The ``HORizontal:DELay`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal:DELay?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal:DELay?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.mode``: The ``HORizontal:DELay:MODe`` command.
        - ``.time``: The ``HORizontal:DELay:TIMe`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._mode = HorizontalDelayMode(device, f"{self._cmd_syntax}:MODe")
        self._time = HorizontalDelayTime(device, f"{self._cmd_syntax}:TIMe")

    @property
    def mode(self) -> HorizontalDelayMode:
        """Return the ``HORizontal:DELay:MODe`` command.

        Description:
            - This command sets or queries the horizontal delay mode.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:DELay:MODe?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:DELay:MODe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:DELay:MODe value``
              command.

        SCPI Syntax:
            ```
            - HORizontal:DELay:MODe {ON|OFF|<NR1>}
            - HORizontal:DELay:MODe?
            ```

        Info:
            - ``OFF`` sets the Horizontal Delay Mode to off. This causes the ``HORizontal:POSition``
              command to operate like the HORIZONTAL POSITION knob on the front panel.
            - ``ON`` sets the Horizontal Delay Mode to on. This causes the ``HORizontal:DELay:TIMe``
              command to operate like the HORIZONTAL POSITION knob on the front panel.
            - ``<NR1>`` = 0 sets the Horizontal Delay Mode to off; any other value sets this mode to
              on.
        """
        return self._mode

    @property
    def time(self) -> HorizontalDelayTime:
        """Return the ``HORizontal:DELay:TIMe`` command.

        Description:
            - This command sets or queries the horizontal delay time that is used when delay mode is
              on.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:DELay:TIMe?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:DELay:TIMe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:DELay:TIMe value``
              command.

        SCPI Syntax:
            ```
            - HORizontal:DELay:TIMe <NR3>
            - HORizontal:DELay:TIMe?
            ```

        Info:
            - ``NR3`` is the delay in seconds.
        """
        return self._time


class Horizontal(SCPICmdRead):
    """The ``HORizontal`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``HORizontal?`` query.
        - Using the ``.verify(value)`` method will send the ``HORizontal?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.delay``: The ``HORizontal:DELay`` command tree.
        - ``.mode``: The ``HORizontal:MODe`` command.
        - ``.position``: The ``HORizontal:POSition`` command.
        - ``.samplerate``: The ``HORizontal:SAMPLERate`` command.
        - ``.scale``: The ``HORizontal:SCAle`` command.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "HORizontal"
    ) -> None:
        super().__init__(device, cmd_syntax)
        self._delay = HorizontalDelay(device, f"{self._cmd_syntax}:DELay")
        self._mode = HorizontalMode(device, f"{self._cmd_syntax}:MODe")
        self._position = HorizontalPosition(device, f"{self._cmd_syntax}:POSition")
        self._samplerate = HorizontalSamplerate(device, f"{self._cmd_syntax}:SAMPLERate")
        self._scale = HorizontalScale(device, f"{self._cmd_syntax}:SCAle")

    @property
    def delay(self) -> HorizontalDelay:
        """Return the ``HORizontal:DELay`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:DELay?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:DELay?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.mode``: The ``HORizontal:DELay:MODe`` command.
            - ``.time``: The ``HORizontal:DELay:TIMe`` command.
        """
        return self._delay

    @property
    def mode(self) -> HorizontalMode:
        """Return the ``HORizontal:MODe`` command.

        Description:
            - This command set or queries the horizontal operating mode.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:MODe?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:MODe?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:MODe value`` command.

        SCPI Syntax:
            ```
            - HORizontal:MODe {AUTO|MANual}
            - HORizontal:MODe?
            ```

        Info:
            - ``AUTO`` selects the automatic horizontal model. Auto mode automatically adjusts the
              sample rate and record length to provide a high acquisition rate in Fast Acq or signal
              fidelity in analysis. Record length is read only.
            - ``MANUAL`` selects the manual horizontal model. Manual mode lets you change the sample
              rate, horizontal scale, and record length. These values interact. For example, when
              you change record length then the horizontal scale also changes.

        Sub-properties:
            - ``.manual``: The ``HORizontal:MODe:MANual`` command tree.
        """
        return self._mode

    @property
    def position(self) -> HorizontalPosition:
        """Return the ``HORizontal:POSition`` command.

        Description:
            - This command sets or queries the horizontal position as a percent of screen width.
              When Horizontal Delay Mode is turned off, this command is equivalent to adjusting the
              HORIZONTAL POSITION knob on the front panel. When Horizontal Delay Mode is turned on,
              the horizontal position is forced to 50%.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:POSition?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:POSition?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:POSition value``
              command.

        SCPI Syntax:
            ```
            - HORizontal:POSition <NR3>
            - HORizontal:POSition?
            ```

        Info:
            - ``<NR3>`` is from 0 to ≈100 and is the position of the trigger point on the screen (0
              = left edge, 100 = right edge).
        """
        return self._position

    @property
    def samplerate(self) -> HorizontalSamplerate:
        """Return the ``HORizontal:SAMPLERate`` command.

        Description:
            - This command sets or queries the horizontal sample rate.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:SAMPLERate?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:SAMPLERate?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:SAMPLERate value``
              command.

        SCPI Syntax:
            ```
            - HORizontal:SAMPLERate <NR3>
            - HORizontal:SAMPLERate?
            ```

        Info:
            - ``<NR3>`` is the horizontal sample rate in samples per second.
        """
        return self._samplerate

    @property
    def scale(self) -> HorizontalScale:
        """Return the ``HORizontal:SCAle`` command.

        Description:
            - This command sets or queries the horizontal scale.

        Usage:
            - Using the ``.query()`` method will send the ``HORizontal:SCAle?`` query.
            - Using the ``.verify(value)`` method will send the ``HORizontal:SCAle?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``HORizontal:SCAle value`` command.

        SCPI Syntax:
            ```
            - HORizontal:SCAle <NR3>
            - HORizontal:SCAle?
            ```

        Info:
            - ``<NR3>`` is the horizontal scale in time per division.
        """
        return self._scale



"""The filesys commands module.

These commands are used in the following models:
TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - FILESYS:COLLECTLOGFILES <QString>
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class FilesysCollectlogfiles(SCPICmdWrite):
    """The ``FILESYS:COLLECTLOGFILES`` command.

    Description:
        - This command copies the internal log files to the local, USB, TekDrive or network drive
          specified by the quoted string argument.

    Usage:
        - Using the ``.write(value)`` method will send the ``FILESYS:COLLECTLOGFILES value``
          command.

    SCPI Syntax:
        ```
        - FILESYS:COLLECTLOGFILES <QString>
        ```

    Info:
        - ``<QString>`` is a quoted string that specifies the file path to save the .zip file to.
    """

    _WRAP_ARG_WITH_QUOTES = True


class Filesys(SCPICmdRead):
    """The ``FILESYS`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``FILESYS?`` query.
        - Using the ``.verify(value)`` method will send the ``FILESYS?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.collectlogfiles``: The ``FILESYS:COLLECTLOGFILES`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "FILESYS") -> None:
        super().__init__(device, cmd_syntax)
        self._collectlogfiles = FilesysCollectlogfiles(
            device, f"{self._cmd_syntax}:COLLECTLOGFILES"
        )

    @property
    def collectlogfiles(self) -> FilesysCollectlogfiles:
        """Return the ``FILESYS:COLLECTLOGFILES`` command.

        Description:
            - This command copies the internal log files to the local, USB, TekDrive or network
              drive specified by the quoted string argument.

        Usage:
            - Using the ``.write(value)`` method will send the ``FILESYS:COLLECTLOGFILES value``
              command.

        SCPI Syntax:
            ```
            - FILESYS:COLLECTLOGFILES <QString>
            ```

        Info:
            - ``<QString>`` is a quoted string that specifies the file path to save the .zip file
              to.
        """
        return self._collectlogfiles
